/*----------------------------------------*/
/* Benchmark Tests for Basic Linear Calc. */
/* Copyright (c) 2004- Tomonori Kouya     */
/*                                        */
/* Version 0.0 : Dec. 8, 2004             */
/*----------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "bnc.h"

void usage(const char *commandname)
{
#ifdef USE_DECIMAL
	printf("USAGE-> %s Dimension MinimamDigits(Decimal) MaximumDigits [Maximum seconds]\n", commandname);
#else
	printf("USAGE-> %s Dimension MinimamDigits(Binary) MaximumDigits [Maximum seconds]\n", commandname);
#endif
}


/* kind of arithmetic */
/* 0 ... Addition of Vectors */
/* 1 ... Scalar Mul. for Vector */
/* 2 ... Inner Production */
/* 3 ... Addition of Square Matrices */
/* 4 ... Scalar Mul. for Matrix */
/* 5 ... Matrix-Vector Mutiplication */
/* 6 ... Matrix-Matrix Multiplication */
int linearbenchloop(int kind_of_arithmetic, long dim, unsigned long int start_prec, unsigned long int end_prec, unsigned long int prec_step, long int maxsec)
{
	long int prec, prec_bin;
#ifdef USE_DECIMAL
	unsigned long int start_bin_prec, end_bin_prec;
#endif
	long int i, j, end_flag = 0;
	double iteration, etime, stime;
	mpf_t tmp, seed5, seed3;
	MPFVector va, vb, vc;
	MPFMatrix ma, mb, mc;

#ifdef USE_DECIMAL
	start_bin_prec = (unsigned long)ceil(start_prec / log10(2.0));
	end_bin_prec   = (unsigned long)ceil(end_prec   / log10(2.0));
	printf("  (%5ld - %5ld bits) :\n", start_bin_prec, end_bin_prec);
	printf("  (%5ld - %5ld dec.d):    sec/op , milli-sec/op , micro-sec/op\n", start_prec, end_prec);
#else
	printf("  (%5ld - %5ld bits) :    sec/op , milli-sec/op , micro-sec/op\n", start_prec, end_prec);
#endif
	for(prec = start_prec; prec <= end_prec; prec *= prec_step)
	{
#ifdef USE_DECIMAL
		prec_bin = (unsigned long)ceil(prec / log10(2.0));
#else
		prec_bin = prec;
#endif
		mpf_init2(tmp, prec_bin);
		mpf_init2(seed5, prec_bin);
		mpf_init2(seed3, prec_bin);

		va = init2_mpfvector(dim, prec_bin);
		vb = init2_mpfvector(dim, prec_bin);
		vc = init2_mpfvector(dim, prec_bin);

		ma = init2_mpfmatrix(dim, dim, prec_bin);
		mb = init2_mpfmatrix(dim, dim, prec_bin);
		mc = init2_mpfmatrix(dim, dim, prec_bin);

		mpf_set_ui(seed5, 5UL); mpf_sqrt(seed5, seed5); // seed = sqrt(5);
		mpf_set_ui(seed3, 3UL); mpf_sqrt(seed3, seed3); // seed = sqrt(3);

		// Initialize va, vb, ma and mb
		for(i = 0; i < dim; i++)
		{
			mpf_mul_ui(tmp, seed5, (unsigned long)(i + 1)); // va_i = i * sqrt(5)
			set_mpfvector_i(va, i, tmp);
			mpf_mul_ui(tmp, seed3, (unsigned long)(dim - i)); // vb_i = (dim - i + 1) * sqrt(3)
			set_mpfvector_i(vb, i, tmp);

			for(j = 0; j < dim; j++)
			{
				mpf_mul_ui(tmp, seed5, (unsigned long)(i + j + 1)); // ma_i = (i + j) * sqrt(5)
				set_mpfmatrix_ij(ma, i, j, tmp);
				mpf_mul_ui(tmp, seed3, (unsigned long)(2 * dim - i - j)); // mb_i = (2 * dim - i - j + 1) * sqrt(3)
				set_mpfmatrix_ij(mb, i, j, tmp);
			}
		}

		// Original Version is mpfr_timings.c@www.mpfr.org
		switch(kind_of_arithmetic)
		{
#ifdef USE_MPFR
			case 0: // Addition of Vectors
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) add_mpfvector(vc, va, vb);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			case 1: // Scalar Mul. for Vector
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) cmul_mpfvector(vc, seed3, va);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			case 2: // Inner Production
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) ip_mpfvector(tmp, va, vb);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			case 3: // Addition of Square Matrices
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) add_mpfmatrix(mc, ma, mb);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			case 4: // Scalar Mul. for Matrix
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) cmul_mpfmatrix(mc, seed3, ma);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			case 5: // Matrix-Vector Mutiplication
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) mul_mpfmatrix_mpfvec(vc, ma, vb);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
			default:
				iteration = 1.0;
				do{
					stime = get_secv();
					for(i = 0; i < iteration; i++) mul_mpfmatrix(mc, ma, mb);
					etime = get_secv() - stime;
					iteration *= 2.0;
				} while(etime < 2);
				break;
#endif
		}

		if(etime > maxsec)
			end_flag = 1;

		etime /= iteration / 2;

		switch(kind_of_arithmetic)
		{
#ifdef USE_MPFR
			case 0: // Addition of Vectors
				printf("[%5ld] va + vb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			case 1: // Scalar Mul. for Vector
				printf("[%5ld] c * va  (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			case 2: // Inner Production
				printf("[%5ld] (va, vb)(%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			case 3: // Addition of Square Matrices
				printf("[%5ld] ma + mb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			case 4: // Scalar Mul. for Matrix
				printf("[%5ld] c * ma  (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			case 5: // Matrix-Vector Mutiplication
				printf("[%5ld] ma * vb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
			default: // Matrix-Matrix Multiplication
				printf("[%5ld] ma * mb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
				break;
#endif
		}

		mpf_clear(tmp);
		mpf_clear(seed5);
		mpf_clear(seed3);

		free_mpfvector(va);
		free_mpfvector(vb);
		free_mpfvector(vc);

		free_mpfmatrix(ma);
		free_mpfmatrix(mb);
		free_mpfmatrix(mc);

		if(end_flag == 1)
			break;
	}

	return end_flag;
}

/* kind of arithmetic */
/* 0 ... Addition of Vectors */
/* 1 ... Scalar Mul. for Vector */
/* 2 ... Inner Production */
/* 3 ... Addition of Square Matrices */
/* 4 ... Scalar Mul. for Matrix */
/* 5 ... Matrix-Vector Mutiplication */
/* 6 ... Matrix-Matrix Multiplication */
int linearbenchloop_double(int kind_of_arithmetic, long dim, long int maxsec)
{
	long int prec;
	long int i, j, end_flag = 0;
	double iteration, etime, stime;
	double tmp, seed5, seed3;
	DVector va, vb, vc;
	DMatrix ma, mb, mc;

	printf("(IEEE double: 53bits)  :    sec/op , milli-sec/op , micro-sec/op\n");

	seed5 = sqrt(5.0);
	seed3 = sqrt(3.0);

	va = init_dvector(dim);
	vb = init_dvector(dim);
	vc = init_dvector(dim);

	ma = init_dmatrix(dim, dim);
	mb = init_dmatrix(dim, dim);
	mc = init_dmatrix(dim, dim);

	// Initialize va, vb, ma and mb
	for(i = 0; i < dim; i++)
	{
		set_dvector_i(va, i, seed5 * (double)(i + 1)); // va_i = i * sqrt(5)
		set_dvector_i(vb, i, (double)(dim - i + 1) * seed3); // vb_i = (dim - i + 1) * sqrt(3)

		for(j = 0; j < dim; j++)
		{
			set_dmatrix_ij(ma, i, j, (double)(i + j + 1) * seed5); // ma_i = (i + j) * sqrt(5)
			set_dmatrix_ij(mb, i, j, (double)(2 * dim - i - j) * seed3);// mb_i = (2 * dim - i - j + 1) * sqrt(3)
		}
	}

	// Original Version is mpfr_timings.c@www.mpfr.org
	switch(kind_of_arithmetic)
	{
		case 0: // Addition of Vectors
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) add_dvector(vc, va, vb);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		case 1: // Scalar Mul. for Vector
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) cmul_dvector(vc, seed3, va);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		case 2: // Inner Production
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) tmp = ip_dvector(va, vb);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		case 3: // Addition of Square Matrices
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) add_dmatrix(mc, ma, mb);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		case 4: // Scalar Mul. for Matrix
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) cmul_dmatrix(mc, seed3, ma);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		case 5: // Matrix-Vector Mutiplication
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) mul_dmatrix_dvec(vc, ma, vb);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
		default:
			iteration = 1.0;
			do{
				stime = get_secv();
				for(i = 0; i < iteration; i++) mul_dmatrix(mc, ma, mb);
				etime = get_secv() - stime;
				iteration *= 2.0;
			} while(etime < 2);
			break;
	}

	if(etime > maxsec)
		end_flag = 1;

	etime /= iteration / 2;

	prec = 53;
	switch(kind_of_arithmetic)
	{
		case 0: // Addition of Vectors
			printf("[%5ld] va + vb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		case 1: // Scalar Mul. for Vector
			printf("[%5ld] c * va  (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		case 2: // Inner Production
			printf("[%5ld] (va, vb)(%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		case 3: // Addition of Square Matrices
			printf("[%5ld] ma + mb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		case 4: // Scalar Mul. for Matrix
			printf("[%5ld] c * ma  (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		case 5: // Matrix-Vector Mutiplication
			printf("[%5ld] ma * vb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
		default: // Matrix-Matrix Multiplication
			printf("[%5ld] ma * mb (%5ld): %10.3g,    %10.3g,    %10.3g\n", dim, prec, etime, etime * 1000, (etime) * 1000 * 1000);
			break;
	}

	free_dvector(va);
	free_dvector(vb);
	free_dvector(vc);

	free_dmatrix(ma);
	free_dmatrix(mb);
	free_dmatrix(mc);

	return end_flag;
}

int main(int argc, char *argv[])
{
	long int dimension, min_prec, max_prec, prec, end_flag;
	long int maxsec = 600;

	if(argc <= 3)
	{
		usage(argv[0]);
		return 0;
	}
	else if(argc >= 4)
	{
		dimension = atol(argv[1]);
		min_prec  = atol(argv[2]);
		max_prec  = atol(argv[3]);

		if(dimension <= 0)
		{
			printf("Invalid Dimension! (%d)\n", dimension);
			return -1;
		}

		if((min_prec < 0) || (min_prec > max_prec))
		{
			printf("Invalid precisions! (min = %ld, max = %ld bits)\n", min_prec, max_prec);
			return -1;
		}
		if(argc >= 4)
		{
			maxsec = atol(argv[3]);
			if(maxsec < 0)
			{
				printf("Invalid Maximum seconds! (%ld seconds)\n", maxsec);
				return -1;
			}
		}
	}

	printf("--------------- Benchmark Test using Basic Linear Calculations -----------------\n");

	linearbenchloop_double(0, dimension, maxsec);
	linearbenchloop_double(1, dimension, maxsec);
	linearbenchloop_double(2, dimension, maxsec);
	linearbenchloop_double(3, dimension, maxsec);
	linearbenchloop_double(4, dimension, maxsec);
	linearbenchloop_double(5, dimension, maxsec);
#ifdef USE_MPFR
#ifdef USE_DECIMAL
	linearbenchloop(0, dimension, min_prec, max_prec, 10, maxsec);
	linearbenchloop(1, dimension, min_prec, max_prec, 10, maxsec);
	linearbenchloop(2, dimension, min_prec, max_prec, 10, maxsec);
	linearbenchloop(3, dimension, min_prec, max_prec, 10, maxsec);
	linearbenchloop(4, dimension, min_prec, max_prec, 10, maxsec);
	linearbenchloop(5, dimension, min_prec, max_prec, 10, maxsec);
#else
	linearbenchloop(0, dimension, min_prec, max_prec, 2, maxsec);
	linearbenchloop(1, dimension, min_prec, max_prec, 2, maxsec);
	linearbenchloop(2, dimension, min_prec, max_prec, 2, maxsec);
	linearbenchloop(3, dimension, min_prec, max_prec, 2, maxsec);
	linearbenchloop(4, dimension, min_prec, max_prec, 2, maxsec);
	linearbenchloop(5, dimension, min_prec, max_prec, 2, maxsec);
#endif
#endif

	printf("------------------------------------ END ---------------------------------------\n");
}

