/*----------------------------------------*/
/* Benchmark Test                         */
/*     using Matrix-matrix Multiplication */
/* Copyright (c) 2004- Tomonori Kouya     */
/*                                        */
/* Version 0.0 : 2011-07-04               */
/*----------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "bnc.h"

// 256k bytes
//#define CACHESIZE (256 * 1024)

// 512k bytes
//#define CACHESIZE (512 * 1024)

// 1M bytes
//#define CACHESIZE (1 * 1024 * 1024)

// 2M bytes
//#define CACHESIZE (2 * 1024 * 1024)

void usage(const char *commandname)
{
	printf("USAGE-> %s precision(in bits) [Maximum seconds]\n", commandname);
}

#ifdef USE_GMP
int matmulloop_mpf(long int start_dim, long int end_dim, long int dim_step, long int maxsec, long int opt_dim_flag, unsigned long prec)
{
	long int dim;
	long int i, j, end_flag;
	double iteration, etime, stime, mflops;
	MPFMatrix da, db, dc;
	mpf_t tmp;

	mpf_init2(tmp, prec);

	for(dim = start_dim; dim <= end_dim; dim += dim_step)
	{
		da = init2_mpfmatrix(dim, dim, prec);
		db = init2_mpfmatrix(dim, dim, prec);
		dc = init2_mpfmatrix(dim, dim, prec);

		for(i = 0; i < dim; i++)
		{
			for(j = 0; j < dim; j++)
			{
//				set_mpfmatrix_ij(da, i, j, (i + j + 1) * sqrt(5.0));
				mpf_sqrt_ui(tmp, 5UL);
				mpf_mul_ui(tmp, tmp, (unsigned long)(i + j + 1));
				set_mpfmatrix_ij(da, i, j, tmp);

//				set_mpfmatrix_ij(db, i, j, (dim - i) * sqrt(3.0));
				mpf_sqrt_ui(tmp, 3UL);
				mpf_mul_ui(tmp, tmp, (unsigned long)(dim - i));
				set_mpfmatrix_ij(db, i, j, tmp);

				mpf_sqrt_ui(tmp, 2UL);
				set_mpfmatrix_ij(dc, i, j, tmp);
			}
		}

		iteration = 1.0;
		do{
			stime = get_secv();
			for(i = 0; i < iteration; i++) mul_mpfmatrix(dc, da, db);
			etime = get_secv() - stime;
			iteration *= 2.0;
		} while(etime < 2);

		if(etime > maxsec)
			end_flag = 1;

		etime /= iteration / 2;

		mflops = (double)dim * (double)dim * (double)dim * 2 / etime / 1000.0 / 1000.0;
		printf("[%5d bits] %5d x %5d: %9.5f sec (%8.5f micro-sec/mul) %f MFlops", prec, dim, dim, etime, (etime / dim / dim / dim) * 1000 * 1000, mflops);
		if(opt_dim_flag == 1)
			printf(" <-- Optimal dimension %+d", dim - (end_dim + start_dim) / 2);
		printf("\n");

		free_mpfmatrix(da);
		free_mpfmatrix(db);
		free_mpfmatrix(dc);
	}

	mpf_clear(tmp);

	return end_flag;
}
#endif

int main(int argc, char *argv[])
{
	long int dim, idim, opt_dim, end_flag;
	long int cachesize = 128, maxsec = 600;
	unsigned long prec;

	if(argc <= 1)
	{
		usage(argv[0]);
		return 0;
	}
	else if(argc >= 2)
	{
		prec = atol(argv[1]);
		if(prec <= 0)
		{
			printf("Invalid precision in bit! (%ld bits)\n", prec);
			return -1;
		}
		if(argc >= 3)
		{
			maxsec = atol(argv[2]);
			if(maxsec < 0)
			{
				printf("Invalid Maximum seconds! (%ld seconds)\n", maxsec);
				return -1;
			}
		}
	}

	printf("------------------ Benchmark Test using Matrix Multiplication -----------------\n");

	//for(idim = 32; idim <= 16384; idim *= 2)
	for(idim = 2; idim <= 2048; idim *= 2)
	{
		end_flag = 0;

		end_flag = matmulloop_mpf(idim - 1, idim + 1, 1, maxsec, 0, prec);

		if((idim <= opt_dim) && (opt_dim <= idim * 2))
			end_flag = matmulloop_mpf(opt_dim - 5, opt_dim + 5, 1, maxsec, 1, prec);

		if(end_flag == 1)
			break;

	}
	printf("------------------------------------ END --------------------------------------\n");
}

