/*----------------------------------------*/
/* Benchmark Test                         */
/*     using Matrix-vector Multiplication */
/* Copyright (c) 2004- Tomonori Kouya     */
/*                                        */
/* Version 0.0 : Dec. 13, 2004            */
/*----------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "bnc.h"

void usage(const char *commandname)
{
	printf("USAGE-> %s MinDimension MaxDimension [Maximum seconds]\n", commandname);
}

int matvecmulloop(long int start_dim, long int end_dim, long int dim_step, long int maxsec, long int opt_dim_flag)
{
	long int dim;
	long int i, j, end_flag;
	double iteration, etime, stime;
	DMatrix da;
	DVector dvx, dvy;

	for(dim = start_dim; dim <= end_dim; dim += dim_step)
	{
		da  = init_dmatrix(dim, dim);
		dvx = init_dvector(dim);
		dvy = init_dvector(dim);

		for(i = 0; i < dim; i++)
		{
			set_dvector_i(dvx, i, (i + 1) * sqrt(3.0));
			for(j = 0; j < dim; j++)
				set_dmatrix_ij(da, i, j, (i + j + 1) * sqrt(5.0));
		}

		iteration = 1.0;
		do{
			stime = get_secv();
			for(i = 0; i < iteration; i++) mul_dmatrix_dvec(dvy, da, dvx);
			etime = get_secv() - stime;
			iteration *= 2.0;
		} while(etime < 2);

		if(etime > maxsec)
			end_flag = 1;

		etime /= iteration / 2;

		printf("%5d x %5d: %9.5f sec (%8.5f micro-sec/mul)", dim, dim, etime, (etime / dim / dim) * 1000 * 1000);
		printf("\n");

		free_dmatrix(da);
		free_dvector(dvx);
		free_dvector(dvy);
	}

	return end_flag;
}

int main(int argc, char *argv[])
{
	long int dim, idim, opt_dim, end_flag;
	long int mindim = 8, maxdim = 65536, maxsec = 600;

	if(argc <= 2)
	{
		usage(argv[0]);
		return 0;
	}
	else if(argc >= 3)
	{
		mindim = atol(argv[1]);
		maxdim = atol(argv[2]);
		if((maxdim <= 0) || (mindim <= 0) || (mindim > maxdim))
		{
			printf("Invalid Dimension! (MinDim = %ld, MaxDim = %ld)\n", mindim, maxdim);
			return -1;
		}
		if(argc >= 4)
		{
			maxsec = atol(argv[3]);
			if(maxsec < 0)
			{
				printf("Invalid Maximum seconds! (%ld seconds)\n", maxsec);
				return -1;
			}
		}
	}

	printf("------------------ Benchmark Test using Matrix-Vector Multiplication -----------\n");

	printf("Minimum limit of dimension : Matrix(%d x %d), Vector(%d)\n", mindim, mindim, mindim);
	printf("Maximum limit of dimension : Matrix(%d x %d), Vector(%d)\n", maxdim, maxdim, maxdim);

	for(idim = mindim; idim <= maxdim; idim += 128)
	{
		end_flag = 0;

		end_flag = matvecmulloop(idim - 1, idim + 1, 1, maxsec, 0);

		if(end_flag == 1)
			break;

	}
	printf("------------------------------------ END --------------------------------------\n");
}

