/********************************************************************************/
/* BNC header file                                                              */
/* Copyright (c) 2000-2012 Tomonori Kouya, All rights reserved.                 */
/*                                                                              */
/* History:                                                                     */
/* Version 0.0.1  2000.5/23: modifying mpf_sin, mpf_cos                         */
/*         0.0.2  2000.5/26: append mpf_ln, mpf_log10                           */
/*         0.0.3  2000.6/13: append FStack, DStack, MPFStack                    */
/*         0.0.4  2000.6/14: append and test euler.c ex_nim.c                   */
/*         0.0.5  2000.6/19: append and test newton.c                           */
/*         0.0.6  2000.6/19: append and test rk.c                               */
/*         0.0.7  2000.6/20: append set0_[f,d,mpf]vector                        */
/*         0.0.7a 2000.6/20: append set0_[f,d,mpf]matrix                        */
/*                           append setI_[f,d,mpf]matrix                        */
/*         0.0.7b 2000.7/04: revise lu.c                                        */
/*         0.0.7c 2000.7/05: revise euler.c, rk.c, rkcoef.c                     */
/*         0.0.7d 2000.12/4: append power.c                                     */
/*         0.0.7e 2001.4/23: append integral.c                                  */
/*         0.0.7f 2001.4/24: append bnc_mpf_[init]_set_d                        */
/*         0.0.7g 2001.6/05: append ivpower.c                                   */
/*         0.1    2001.8/02: setting precision for                              */
/*                           MPF* functions                                     */
/*         0.1a   2001.8/08: Bug Fix : init_*vector[2],                         */
/*                           rkcoef.c, ex_nim.c                                 */
/*         0.2    2002.5/04: append poly.c, complex.c,                          */
/*                           array, dka.c                                       */
/*         0.2a   2002.5/28: append iterative.c                                 */
/*         0.3    2002.9/05: include mpfr                                       */
/*         0.3a   2003.2/27: append get_sec.c & bug fix                         */
/*         0.3b   2003.2/27: separate modules of ODEs                           */
/*         0.3c   2003.6/11: append qr.c & gtestmat.c                           */
/*         0.3d   2003.12/6: append get_real_sec()                              */
/*         0.3e   2004.1/07: include new functions                              */
/*                                       from mpfr-2.0.2                        */
/*         0.3f   2004.3/06: append xmtrapezoidal_fs funcs                      */
/*                                       into integral.c                        */
/*         0.4    2004.3/08: append rational.c                                  */
/*         0.4a   2004.9/18: modifiy linear.c with mpfr_fma                     */
/*         0.4b   2004.11/3: append krylov.c                                    */
/*         0.5    2005.6/07: append diff.c                                      */
/*         0.5a   2005.6/07: modify gtestmat.c                                  */
/*         0.6    2005.7/12: append fread_write.c                               */
/*         0.6a   2005.7/12: modify linear.c(copy_*vector_ij)                   */
/*         0.6b   2006.1/13: modify lu.c(True LU decomposion)                   */
/*         0.6b1  2006.1/26: append mul_*poly in poly.c                         */
/*                2006.1/26: append mpf_fma in efunc.c                          */
/*         0.6b2  2006.2/13: append norms of matrix                             */
/*         0.6b2  2006.3/14: Bug fix : mpf_max, min                             */
/*         0.6b3  2006.7/26: modify diff.c                                      */
/*         0.6b4  2006.10/13: Bug fix : macro in bnc.h                          */
/*         0.6b5  2006.10/13: append mharmonic.c                                */
/*         0.6b6  2007.01/11: Bug fix : poly.c                                  */
/*         0.6b7  2007.06/26: Append algebraic_eq.c                             */
/*         0.6b8  2008.06/04: Bug fix : complex.c, array.c                      */
/*         0.6c   2008.06/04: Bug fix : Makefile (Thanks, Elena)                */
/*         0.6d   2011.01/31: Fix MPI functions for mpfr 3.x                    */
/*         0.7    2011.08/25: append bncsparse                                  */
/*         0.79   2012-03-18: append clinear.c clu.c                            */
/*                            modify cg.c, krylov.c                             */
/*         0.791  2012-06-03: Bug fix: linear.c linear_append.c poly.c          */
/*         0.792  2012-07-05: append bmatrix.c                                  */
/*         0.7921 2012-07-13: modify readmatrix.c & bug fix                     */
/*         0.7922 2012-07-18: Buf fix: lu.c, bmatrix.c, poly.c                  */
/*         0.8    2013-02-28: Append print_date                                 */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
#ifndef __STDIO_H__
#include <stdio.h>
#endif

#ifndef __STDLIB_H__
#include <stdlib.h>
#endif

#ifndef __MATH_H__
#include <math.h>
#endif

#ifndef __COMPLEX_H__
#include <complex.h> // C99 complex arithmetic
#endif

#ifndef M_PI
#define M_PI (3.14159265358979323846264)
#endif

#ifndef __BNC_H
#define __BNC_H

#define BNC_ERROR   (-1)
#define BNC_SUCCESS (0)

#ifdef USE_GMP
  #ifndef __GMP_H
    #include "gmp.h"
//    #include "gmp-impl.h"
  #endif
  static unsigned long int bnc_default_prec = 128; /* Global variable */
  #ifdef USE_MPFR
    #ifndef __MPFR_H
      #include "mpfr.h"
    #endif // fix! 2006-10-13
//      #include "mpfr-impl.h"
    #include "_bnc_mpf.h" // mpf_* -> _bnc_mpf_*
    #include "mpf2mpfr.h"
// check MPFR's Version 
    #ifdef mpfr_cmp_abs // over 2.0.2
      #define _BNC_MPFR_VER 202
    #else
	  #define _BNC_MPFR_VER 201 // mpfr in gmp-4.1.2
    #endif
    #define mpf_ptr mpfr_ptr
    static unsigned long int bnc_default_rounding_mode = GMP_RNDN; /* Global variable */
  #endif
#endif

#if defined (__cplusplus)
extern "C" {
#endif
/*************************************************/
/* double get_sec(int flag)                      */
/*************************************************/
double get_sec(int);
double get_secv(void);
float fget_sec(int);
float fget_secv(void);

double get_real_sec(int);
double get_real_secv(void);
float fget_real_sec(int);
float fget_real_secv(void);
void print_date(void);

/*************************************************/
/* Complex -> Cmplx                              */
/* FCmplx, DCmplx, MPFCmplx                      */
/*************************************************/
typedef struct{
	float re;
	float im;
} fcmplx;

typedef fcmplx *FCmplx;

typedef struct{
	double re;
	double im;
} dcmplx;

typedef dcmplx *DCmplx;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t re;
	mpf_t im;
} mpfcmplx;

typedef mpfcmplx *MPFCmplx;
#endif

/*************************************************/
/* Polynomial Type: FPoly, DPoly, MPFPoly        */
/*                  CFPoly, CDPoly, CMPFPoly     */
/*************************************************/
typedef struct{
	float *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} fpoly;

typedef fpoly *FPoly;

typedef struct{
	fcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cfpoly;

typedef cfpoly *CFPoly;

typedef struct{
	double *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} dpoly;

typedef dpoly *DPoly;

typedef struct{
	dcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cdpoly;

typedef cdpoly *CDPoly;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t *coef;
	mpf_t zero; /* ... + 0 x^(deg+2) + 0 x^(deg+1) + coef[i] x^deg + ... */
//	mpf_ptr *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} mpfpoly;

typedef mpfpoly *MPFPoly;

typedef struct{
	unsigned long int prec;
	mpfcmplx *coef;
	long int deg;
	long int max_len; /* coef[max_len] */
} cmpfpoly;

typedef cmpfpoly *CMPFPoly;
#endif

/*************************************************/
/* Vector Type: FVector, DVector, MPFVector      */
/*              CFVector, CDVector, CMPFVector   */
/*************************************************/
typedef struct{
	float *element;
	long int dim;
} fvector;

typedef fvector *FVector;

typedef struct{
	fcmplx *element;
	long int dim;
} cfvector;

typedef cfvector *CFVector;

typedef struct{
	double *element;
	long int dim;
} dvector;

typedef dvector *DVector;

/* obsolete define: 2012-03-18
 typedef struct{
	dcmplx *element;
	long int dim;
} cdvector;

typedef cdvector *CDVector;
*/

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim;
} mpfvector;

typedef mpfvector *MPFVector;

/* obsolete: 2012-03-18
typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int dim;
} cmpfvector;

typedef cmpfvector *CMPFVector;
*/

typedef struct{
	mpq_t *element;
	long int dim;
} mpqvector;

typedef mpqvector *MPQVector;

#endif

/*************************************************/
/* Matrix Type: FMatrix, DMatrix, MPFMatrix      */
/*              CFMatrix, CDMatrix, CMPFMatrix   */
/*************************************************/
#define MTYPE_DEFAULT		0x3F /* 0011 1111 square, unsymmetric, dense */
#define MTYPE_GENERAL		0x00 /*        00 */
#define MTYPE_SQUARE		0x03 /*        11 */
#define MTYPE_SYMMETRIC		0x00 /*      00   */
#define MTYPE_UNSYMMETRIC	0x0C /*      11   */
#define MTYPE_SPARSE		0x00 /*   00      */
#define MTYPE_BAND		0x20 /*   10      */
#define MTYPE_DENSE		0x30 /*   11      */

typedef struct{
//	unsigned int type; /* Matrix Type */
	float *element;	   /* Elements */
//	long int element_size; /* Number of Elements */
	long int row_dim, col_dim; /* Number of Rows, Number of Columns */
//	long int *row, *col; /* Coordinates of ALL Elements */
} fmatrix;

typedef fmatrix *FMatrix;

typedef struct{
	fcmplx *element;	   /* Elements */
	long int row_dim, col_dim; /* Number of Rows, Number of Columns */
} cfmatrix;

typedef cfmatrix *CFMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int row_dim, col_dim;
} dmatrix;

typedef dmatrix *DMatrix;

/* obsolete define: 2012-03-18
typedef struct{
	dcmplx *element;
	long int row_dim, col_dim;
} cdmatrix;

typedef cdmatrix *CDMatrix;
*/

#ifdef USE_GMP

typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int row_dim, col_dim;
} mpfmatrix;

typedef mpfmatrix *MPFMatrix;

/*
typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int row_dim, col_dim;
} cmpfmatrix;

typedef cmpfmatrix *CMPFMatrix;
*/

typedef struct{
//	unsigned int type;
	mpq_t *element;
	long int row_dim, col_dim;
} mpqmatrix;

typedef mpqmatrix *MPQMatrix;
#endif

/*************************************************/
/* Symmetric Matrix Type:                        */
/*              FSMatrix, DSMatrix, MPFSMatrix   */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim;
} fsmatrix;

typedef fmatrix *FSMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim;
} dsmatrix;

typedef dmatrix *DSMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim;
} mpfsmatrix;

typedef mpfmatrix *MPFSMatrix;
#endif

/*************************************************/
/* Band Matrix Type:                             */
/*              FBMatrix, DBMatrix, MPFBMatrix   */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim, upper_dim, lower_dim;
} fbmatrix;

typedef fbmatrix *FBMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim, upper_dim, lower_dim;
} dbmatrix;

typedef dbmatrix *DBMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim, upper_dim, lower_dim;
	mpf_t zero; // = 0
} mpfbmatrix;

typedef mpfbmatrix *MPFBMatrix;
#endif

/*************************************************/
/* Symmetric Band Matrix Type:                   */
/*             FSBMatrix, DSBMatrix, MPFSBMatrix */
/*************************************************/
typedef struct{
//	unsigned int type;
	float *element;
	long int dim, band_width;
} fsbmatrix;

typedef fsbmatrix *FSBMatrix;

typedef struct{
//	unsigned int type;
	double *element;
	long int dim, band_width;
} dsbmatrix;

typedef dsbmatrix *DSBMatrix;

#ifdef USE_GMP
typedef struct{
	unsigned long int prec;
//	unsigned int type;
	mpf_t *element;
//	mpf_ptr *element;
	long int dim, band_width;
} mpfsbmatrix;

typedef mpfsbmatrix *MPFSBMatrix;
#endif

/*************************************************/
/* Stack                                         */
/*************************************************/
/* fstack */
typedef struct {
	float *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} fstack;

typedef fstack *FStack;

/* dstack */
typedef struct {
	double *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} dstack;

typedef dstack *DStack;

#ifdef USE_GMP
/* mpfstack */
typedef struct {
	unsigned long prec; /* precision */
	mpf_t *array; /* stack */
	long int size; /* Height of stack */
	long index; /* pointer to top of stack */
} mpfstack;

typedef mpfstack *MPFStack;
#endif

/* errors */
// #define STACK_OVERFLOW	10
// #define STACK_UNDERFLOW	20

/*************************************************/
/* Array                                         */
/*************************************************/
/* farray */
typedef struct {
	float *array; /* array */
	long int size; /* Length of array */
} farray;

typedef farray *FArray;

/* cfarray */
typedef struct {
	fcmplx *array; /* array */
	long int size; /* Length of array */
} cfarray;

typedef cfarray *CFArray;

/* darray */
typedef struct {
	double *array; /* array */
	long int size; /* Length of array */
} darray;

typedef darray *DArray;

/* cdarray */
typedef struct {
	dcmplx *array; /* array */
	long int size; /* Length of array */
} cdarray;

typedef cdarray *CDArray;

#ifdef USE_GMP
/* mpfarray */
typedef struct {
	unsigned long prec; /* preision */
	mpf_t *array; /* array */
	long int size; /* Length of array */
} mpfarray;

typedef mpfarray *MPFArray;

/* mpfarray */
typedef struct {
	unsigned long prec; /* preision */
	mpfcmplx *array; /* array */
	long int size; /* Length of array */
} cmpfarray;

typedef cmpfarray *CMPFArray;
#endif

/*************************************************/
/* efunc.c: Elementary Functions                 */
/*************************************************/
long int lmax(long, long);
long int lmin(long, long);
long int lfact(long, long);
#if __GNUC__ < 3
float fmax(float, float);
float fmin(float, float);
#endif
float fpower(float, long);
double dmax(double, double);
double dmin(double, double);
double dpower(double, long);
#ifdef USE_GMP
void set_bnc_default_prec(unsigned long);
void set_bnc_default_prec_decimal(unsigned long);
void _set_bnc_default_prec(unsigned long);
void _set_bnc_default_prec_decimal(unsigned long);
unsigned long get_bnc_default_prec(void);
#ifdef USE_MPFR
  void set_bnc_rounding_mode(mp_rnd_t);
  void _set_bnc_rounding_mode(mp_rnd_t);
  mp_rnd_t get_bnc_default_rounding_mode(void);
#endif
void mpf_power(mpf_t, mpf_t, long);
mpf_ptr mpf_max(mpf_t, mpf_t);
mpf_ptr mpf_min(mpf_t, mpf_t);
void bnc_mpf_set_d(mpf_t, double);
void bnc_mpf_init_set_d(mpf_t, double);
void mpf_fma(mpf_t, mpf_t, mpf_t, mpf_t);
#if (__GNU_MP_VERSION < 3)
void mpf_floor(mpf_t, mpf_t);
#else
  #ifdef USE_MPFR
    #define mpf2float(x) (float)mpfr_get_d(x, bnc_default_rounding_mode)
    #define mpf2double(x) mpfr_get_d(x, bnc_default_rounding_mode)
  #else
    float mpf2float(mpf_t);
    double mpf2double(mpf_t);
  #endif
#endif
#ifdef USE_MPFR
  #define mpf_pi(x) mpfr_const_pi(x, bnc_default_rounding_mode)
  void mpf_e(mpfr_t);
  //void mpfr_get_f(mpf_t, mpfr_t, mp_rnd_t);
  void mpfr_get_f_bnc(mpf_t, mpfr_t, mp_rnd_t);
  #define mpf_sin(x, y) mpfr_sin(x, y, bnc_default_rounding_mode)
  #define mpf_cos(x, y) mpfr_cos(x, y, bnc_default_rounding_mode)
  #define mpf_tan(x, y) mpfr_tan(x, y, bnc_default_rounding_mode)
  #define mpf_asin(x, y) mpfr_asin(x, y, bnc_default_rounding_mode)
  #define mpf_acos(x, y) mpfr_acos(x, y, bnc_default_rounding_mode)
  #define mpf_atan(x, y) mpfr_atan(x, y, bnc_default_rounding_mode)
  #define mpf_exp(x, y) mpfr_exp(x, y, bnc_default_rounding_mode)
  #define mpf_ln_2(x) mpfr_const_log2(x, bnc_default_rounding_mode)
  #define mpf_ln(x, y) mpfr_log(x, y, bnc_default_rounding_mode)
  #define mpf_log10(x, y) mpfr_log10(x, y, bnc_default_rounding_mode)
  #define mpf_euler(x) mpfr_const_euler(x, bnc_default_rounding_mode)
  #if ( _BNC_MPFR_VER > 201 )
      #define mpf_cbrt(x, y) mpfr_cbrt(x, y, bnc_default_rounding_mode)
      #define mpf_sin_cos(x, y, z) mpfr_sin_cos(x, y, z, bnc_default_rounding_mode)
      #define mpf_expm1(x, y) mpfr_expm1(x, y, bnc_default_rounding_mode)
      #define mpf_gamma(x, y) mpfr_gamma(x, y, bnc_default_rounding_mode)
      #define mpf_zeta(x, y) mpfr_zeta(x, y, bnc_default_rounding_mode)
      #define mpf_erf(x, y) mpfr_erf(x, y, bnc_default_rounding_mode)
  #endif  
#else
  void mpf_pi(mpf_t);
  void mpf_e(mpf_t);
  void mpf_sin(mpf_t, mpf_t);
  void mpf_cos(mpf_t, mpf_t);
  void mpf_exp(mpf_t, mpf_t);
  void mpf_ln_2(mpf_t);
  void mpf_ln(mpf_t, mpf_t);
  void mpf_log10(mpf_t, mpf_t);
#endif
#endif

/*************************************************/
/* stack.c: Stack                                */
/*************************************************/
FStack init_fstack(long int stack_size);
void free_fstack(FStack st);
void push_fstack(FStack st, float val);
float pop_fstack(FStack st);
DStack init_dstack(long int stack_size);
void free_dstack(DStack st);
void push_dstack(DStack st, double val);
double pop_dstack(DStack st);
#ifdef USE_GMP
MPFStack init_mpfstack(long int stack_size);
MPFStack init2_mpfstack(long int stack_size, unsigned long prec);
void free_mpfstack(MPFStack st);
void push_mpfstack(MPFStack st, mpf_t val);
void pop_mpfstack(mpf_t rval, MPFStack st);
#endif

/*************************************************/
/* array.c: array                                */
/*************************************************/
FArray init_farray(long int);
void free_farray(FArray);
float get_farray_i(FArray, long int);
void set_farray_i(FArray, long int, float);
void subst_farray(FArray, FArray);
void print_farray(FArray);

CFArray init_cfarray(long int);
void free_cfarray(CFArray);
FCmplx get_cfarray_i(CFArray, long int);
void set_cfarray_i(CFArray, long int, FCmplx);
void subst_cfarray(CFArray, CFArray);
void print_cfarray(CFArray);

DArray init_darray(long int);
void free_darray(DArray);
double get_darray_i(DArray, long int);
void set_darray_i(DArray, long int, double);
void subst_darray(DArray, DArray);
void print_darray(DArray);

CDArray init_cdarray(long int);
void free_cdarray(CDArray);
DCmplx get_cdarray_i(CDArray, long int);
void set_cdarray_i(CDArray, long int, DCmplx);
void subst_cdarray(CDArray, CDArray);
void print_cdarray(CDArray);

#ifdef USE_GMP
MPFArray init_mpfarray(long int);
MPFArray init2_mpfarray(long int, unsigned long);
void free_mpfarray(MPFArray);
mpf_ptr get_mpfarray_i(MPFArray, long int);
void set_mpfarray_i(MPFArray, long int, mpf_t);
void subst_mpfarray(MPFArray, MPFArray);
void print_mpfarray(MPFArray);

CMPFArray init_cmpfarray(long int);
CMPFArray init2_cmpfarray(long int, unsigned long);
void free_cmpfarray(CMPFArray);
MPFCmplx get_cmpfarray_i(CMPFArray, long int);
void set_cmpfarray_i(CMPFArray, long int, MPFCmplx);
void subst_cmpfarray(CMPFArray, CMPFArray);
void print_cmpfarray(CMPFArray);
#endif

/*************************************************/
/* linear.c: Function for Vector Types           */
/*************************************************/

/* Initialize:                                   */
FVector init_fvector(long int dimension);
DVector init_dvector(long int dimension);
#ifdef USE_GMP
MPFVector init_mpfvector(long int dimension);
MPFVector init2_mpfvector(long int dimension, unsigned long);
#endif

/* Free:                                         */
void free_fvector(FVector vec);
void free_dvector(DVector vec);
#ifdef USE_GMP
void free_mpfvector(MPFVector vec);
#endif

/* Get & Set Values:                             */
#define gfvi get_fvector_i
#define gdvi get_dvector_i
float get_fvector_i(FVector vec, long int index);
double get_dvector_i(DVector vec, long int index);
#ifdef USE_GMP
#define gmpfvi get_mpfvector_i
mpf_ptr get_mpfvector_i(MPFVector vec, long int index);
#endif
#define sfvi set_fvector_i
#define sdvi set_dvector_i
void set_fvector_i(FVector vec, long int index, float val);
void set_dvector_i(DVector vec, long int index, double val);
#ifdef USE_GMP
#define smpfvi set_mpfvector_i
#define smpfvid set_mpfvector_i_d
#define smpfvis set_mpfvector_i_str
#define smpfviui set_mpfvector_i_ui
void set_mpfvector_i(MPFVector vec, long int index, mpf_t val);
void set_mpfvector_i_d(MPFVector vec, long int index, double val);
void set_mpfvector_i_str(MPFVector vec, long int index, const char *str, int base);
void set_mpfvector_i_ui(MPFVector vec, long int index, unsigned long val);
#endif

/* For MPFVector */
#ifdef USE_GMP
unsigned long int prec_mpfvector(MPFVector);
unsigned long int minprec_mpfvector(MPFVector);
unsigned long int maxprec_mpfvector(MPFVector);
#endif

/* Output:                                       */
void print_fvector(FVector);
void print_dvector(DVector);
void print_fdvector(FVector, DVector);
#ifdef USE_GMP
void print_mpfvector(MPFVector);
void print_fdmpfvector(FVector, DVector, MPFVector);
#endif

/*************************************************/
/* linear.c: Function for Matrix Types           */
/*************************************************/

/* Initialize:                                   */
FMatrix init_fmatrix(long int row_dimension, long int col_dimension);
DMatrix init_dmatrix(long int row_dimension, long int col_dimension);
#ifdef USE_GMP
MPFMatrix init_mpfmatrix(long int row_dimension, long int col_dimension);
MPFMatrix init2_mpfmatrix(long int row_dimension, long int col_dimension, unsigned long mbits);
#endif

/* Free:                                         */
void free_fmatrix(FMatrix mat);
void free_dmatrix(DMatrix mat);
#ifdef USE_GMP
void free_mpfmatrix(MPFMatrix mat);
#endif

/* Get & Set:                                    */
#define gfmij get_fmatrix_ij
#define gdmij get_dmatrix_ij
float get_fmatrix_ij(FMatrix mat, long int row_index, long int col_index);
double get_dmatrix_ij(DMatrix mat, long int row_index, long int col_index);
#ifdef USE_GMP
#define gmpfmij get_mpfmatrix_ij
mpf_ptr get_mpfmatrix_ij(MPFMatrix mat, long int row_index, long int col_index);
#endif
#define sfmij set_fmatrix_ij
#define sdmij set_dmatrix_ij
void set_fmatrix_ij(FMatrix mat, long int row_index, long int col_index, float val);
void set_dmatrix_ij(DMatrix mat, long int row_index, long int col_index, double val);
#ifdef USE_GMP
#define smpfmij set_mpfmatrix_ij
#define smpfmijd set_mpfmatrix_ij_d
#define smpfmijs set_mpfmatrix_ij_str
#define smpfmijui set_mpfmatrix_ij_ui
void set_mpfmatrix_ij(MPFMatrix mat, long int row_index, long int col_index, mpf_t val);
void set_mpfmatrix_ij_d(MPFMatrix mat, long int row_index, long int col_index, double val);
void set_mpfmatrix_ij_str(MPFMatrix mat, long int row_index, long int col_index, const char *str, int base);
void set_mpfmatrix_ij_ui(MPFMatrix mat, long int row_index, long int col_index, const unsigned long val);
#endif

/* For MPFVector */
#ifdef USE_GMP
unsigned long int prec_mpfmatrix(MPFMatrix);
unsigned long int minprec_mpfmatrix(MPFMatrix);
unsigned long int maxprec_mpfmatrix(MPFMatrix);
#endif

/* Output:                                       */
void print_fmatrix(FMatrix mat);
void print_dmatrix(DMatrix mat);
#ifdef USE_GMP
void print_mpfmatrix(MPFMatrix mat);
#endif


/*************************************************/
/* linear.c: Elementary Arithmetic for Vector    */
/*************************************************/
/* For FVector */
void add_fvector(FVector, FVector, FVector);
void add2_fvector(FVector, FVector);
void sub_fvector(FVector, FVector, FVector);
void sub2_fvector(FVector, FVector);
void cmul_fvector(FVector, float, FVector);
void cmul2_fvector(FVector, float);
float ip_fvector(FVector, FVector);
float norm1_fvector(FVector);
float norm2_fvector(FVector);
float normi_fvector(FVector);
void subst_fvector(FVector, FVector);

/* For DVector */
void add_dvector(DVector, DVector, DVector);
void add2_dvector(DVector, DVector);
void sub_dvector(DVector, DVector, DVector);
void sub2_dvector(DVector, DVector);
void cmul_dvector(DVector, double, DVector);
void cmul2_dvector(DVector, double);
void add_cmul_dvector(DVector, DVector, double, DVector);
double ip_dvector(DVector, DVector);
double norm1_dvector(DVector);
double norm2_dvector(DVector);
double normi_dvector(DVector);
void subst_dvector(DVector, DVector);
void copy_dvector_ij(DVector, long int, long int, DVector, long int, long int);

/* get residual in double precision */
/* r := b - A * x */
void residual_dmat_dvec(DVector, DVector, DMatrix, DVector);

/* vec := -vec */
void neg_dvector(DVector);

#ifndef BNC_VECTOR_NORMI
  #define BNC_VECTOR_NORMI 0 // Infinity Norm
#endif
#ifndef BNC_VECTOR_NORM1
  #define BNC_VECTOR_NORM1 1 // 1-Norm
#endif
#ifndef BNC_VECTOR_NORM2
  #define BNC_VECTOR_NORM2 2 // Euclid Norm(default)
#endif

/* Elementwise relative errors of vector */
void relerr_element_dvector(double *, double *, double *, DVector, DVector, int);

/* --- F and D mixed operation --- */

/* c := (double)a */
void subst_fvector_dvec(FVector, DVector);

/* (double)c := (float)a */
void subst_dvector_fvec(DVector, FVector);

/* For MPFVector */
#ifdef USE_GMP
void add_mpfvector(MPFVector, MPFVector, MPFVector);
void add2_mpfvector(MPFVector, MPFVector);
void sub_mpfvector(MPFVector, MPFVector, MPFVector);
void sub2_mpfvector(MPFVector, MPFVector);
void cmul_mpfvector(MPFVector, mpf_t, MPFVector);
void cmul2_mpfvector(MPFVector, mpf_t);
void add_cmul_mpfvector(MPFVector, MPFVector, mpf_t, MPFVector);
void ip_mpfvector(mpf_t, MPFVector, MPFVector);
void norm1_mpfvector(mpf_t, MPFVector);
void norm2_mpfvector(mpf_t, MPFVector);
void normi_mpfvector(mpf_t, MPFVector);
void subst_mpfvector(MPFVector, MPFVector);
void copy_mpfvector_ij(MPFVector, long int, long int, MPFVector, long int, long int);

/* c := (double)a */
void subst_dvector_mpfvec(DVector, MPFVector);

/* (mpf_t)c := (double)a */
void subst_mpfvector_dvec(MPFVector, DVector);

/* vec := -vec */
void neg_mpfvector(MPFVector);

/* Normwise relative error of vector */
void relerr_mpfvector(mpf_t, MPFVector, MPFVector, int);

/* Elementwise relative errors of vector */
void relerr_element_mpfvector(mpf_t, mpf_t, mpf_t, MPFVector, MPFVector, int);
#endif

/*************************************************/
/* linear.c: Elementary Arithmetic for Matrices  */
/*************************************************/
/* For FMatrix */
void add_fmatrix(FMatrix, FMatrix, FMatrix);
void sub_fmatrix(FMatrix, FMatrix, FMatrix);
void subst_fmatrix(FMatrix, FMatrix);
void set0_fmatrix(FMatrix);
void setI_fmatrix(FMatrix);
void cmul_fmatrix(FMatrix, float, FMatrix);
void mul_fmatrix(FMatrix, FMatrix, FMatrix);
void mul_fmatrix_fvec(FVector, FMatrix, FVector);
void transpose_fmatrix(FMatrix, FMatrix);
void inv_fmatrix(FMatrix);

/* For DMatrix */
double normf_dmatrix(DMatrix);
double normi_dmatrix(DMatrix);
double norm1_dmatrix(DMatrix);
void add_dmatrix(DMatrix, DMatrix, DMatrix);
void sub_dmatrix(DMatrix, DMatrix, DMatrix);
void subst_dmatrix(DMatrix, DMatrix);
void set0_dmatrix(DMatrix);
void setI_dmatrix(DMatrix);
void cmul_dmatrix(DMatrix, double, DMatrix);
void mul_dmatrix(DMatrix, DMatrix, DMatrix);
void mul_dmatrix_dvec(DVector, DMatrix, DVector);
void mul_dmatrixt_dvec(DVector, DMatrix, DVector);
void transpose_dmatrix(DMatrix, DMatrix);
void inv_dmatrix(DMatrix);

/* c := (double)a */
void subst_fmatrix_dmat(FMatrix, DMatrix);

/* c := (double)a^T */
void subst_fmatrix_dmat_trans(FMatrix, DMatrix);

/* For MPFMatrix */
#ifdef USE_GMP
void normf_mpfmatrix(mpf_t, MPFMatrix);
void norm1_mpfmatrix(mpf_t, MPFMatrix);
void normi_mpfmatrix(mpf_t, MPFMatrix);
void add_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void sub_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void subst_mpfmatrix(MPFMatrix, MPFMatrix);
void set0_mpfmatrix(MPFMatrix);
void setI_mpfmatrix(MPFMatrix);
void cmul_mpfmatrix(MPFMatrix, mpf_t, MPFMatrix);
void mul_mpfmatrix(MPFMatrix, MPFMatrix, MPFMatrix);
void mul_mpfmatrix_mpfvec(MPFVector, MPFMatrix, MPFVector);
void mul_mpfmatrixt_mpfvec(MPFVector, MPFMatrix, MPFVector);
void transpose_mpfmatrix(MPFMatrix, MPFMatrix);
void inv_mpfmatrix(MPFMatrix);

/* get residual in mpf_t precision */
/* r := b - A * x */
void residual_mpfmat_mpfvec(MPFVector, MPFVector, MPFMatrix, MPFVector);

/* --- D and M mixed operation --- */

/* c := (mpf_t)a */
void subst_dmatrix_mpfmat(DMatrix, MPFMatrix);

/* c := (mpf_t)a^T */
void subst_dmatrix_mpfmat_trans(DMatrix, MPFMatrix);

#endif

/*************************************************/
/* Linear System                                 */
/*************************************************/
/* LU decompositions & Backward Substitutions */
int FLUdecomp(FMatrix);
int SolveFLS(FVector, FMatrix, FVector);
int FLUdecompP(FMatrix, long[]);
int SolveFLSP(FVector, FMatrix, FVector, long[]);
int FLUdecompC(FMatrix, long[], long[]);
int SolveFLSC(FVector, FMatrix, FVector, long[], long[]);
int DLUdecomp(DMatrix);
int SolveDLS(DVector, DMatrix, DVector);
int DLUdecompP(DMatrix, long[]);
int SolveDLSP(DVector, DMatrix, DVector, long[]);
int DLUdecompC(DMatrix, long[], long[]);
int SolveDLSC(DVector, DMatrix, DVector, long[], long[]);
#ifdef USE_GMP
int MPFLUdecomp(MPFMatrix);
int SolveMPFLS(MPFVector, MPFMatrix, MPFVector);
int MPFLUdecompP(MPFMatrix, long[]);
int SolveMPFLSP(MPFVector, MPFMatrix, MPFVector, long[]);
int MPFLUdecompC(MPFMatrix, long[], long[]);
int SolveMPFLSC(MPFVector, MPFMatrix, MPFVector, long[], long[]);
#endif

/*************************************************/
/* cg.c: Conjugate-Gradient Methods              */
/*************************************************/
long int FCG(FVector, FMatrix, FVector, float, float, long);
long int DCG(DVector, DMatrix, DVector, double, double, long);
#ifdef USE_GMP
long int MPFCG(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
#endif

/*************************************************/
/* power.c, ivpower.c: Power and Inverse Power Methods */
/*************************************************/
long int fpower_eig(float *, FVector, FMatrix, FVector, float, float, long);
long int fivpower_eig(float *, FVector, FMatrix, FVector, float, float, long);
long int dpower_eig(double *, DVector, DMatrix, DVector, double, double, long);
long int divpower_eig(double *, DVector, DMatrix, DVector, double, double, long);
#ifdef USE_GMP
long int mpf_power_eig(mpf_t, MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
long int mpf_ivpower_eig(mpf_t, MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
#endif

/*************************************************/
/* newton.c: Newton/Simplified Newton Methods    */
/*************************************************/
long int fnewton_1(float *, float, float (* func)(float), float (* dfunc)(float), long int, float, float);
long int fsnewton_1(float *, float, float (* func)(float), float (* dfunc)(float), long int, float, float);
long int fnewton(FVector, FVector, void (* func)(FVector, FVector), void (* jfunc)(FMatrix, FVector), long int, float, float);
long int fsnewton(FVector, FVector, void (* func)(FVector, FVector), void (* jfunc)(FMatrix, FVector), long int, float, float);
long int dnewton_1(double *, double, double (* func)(double), double (* dfunc)(double), long int, double, double);
long int dsnewton_1(double *, double, double (* func)(double), double (* dfunc)(double), long int, double, double);
long int dnewton(DVector, DVector, void (* func)(DVector, DVector), void (* jfunc)(DMatrix, DVector), long int, double, double);
long int dsnewton(DVector, DVector, void (* func)(DVector, DVector), void (* jfunc)(DMatrix, DVector), long int, double, double);
#ifdef USE_GMP
long int mpf_newton_1(mpf_t, mpf_t, void (* func)(mpf_t, mpf_t), void (* dfunc)(mpf_t, mpf_t), long int, mpf_t, mpf_t);
long int mpf_snewton_1(mpf_t, mpf_t, void (* func)(mpf_t, mpf_t), void (* dfunc)(mpf_t, mpf_t), long int, mpf_t, mpf_t);
long int mpf_newton(MPFVector, MPFVector, void (* func)(MPFVector, MPFVector), void (* jfunc)(MPFMatrix, MPFVector), long int, mpf_t, mpf_t);
long int mpf_snewton(MPFVector, MPFVector, void (* func)(MPFVector, MPFVector), void (* jfunc)(MPFMatrix, MPFVector), long int, mpf_t, mpf_t);
#endif

/*************************************************/
/* poly.c: Algebraic equations and Polynomial    */
/*************************************************/
FPoly init_fpoly(long int);
DPoly init_dpoly(long int);
void free_fpoly(FPoly);
void free_dpoly(DPoly);
float get_fpoly_i(FPoly, long int);
double get_dpoly_i(DPoly, long int);
long int setdegree_fpoly(FPoly);
long int setdegree_dpoly(DPoly);
void set_fpoly_i(FPoly, long int, float);
void set_dpoly_i(DPoly, long int, double);
void print_fpoly(FPoly);
void print_dpoly(DPoly);
void add_fpoly(FPoly, FPoly, FPoly);
void sub_fpoly(FPoly, FPoly, FPoly);
void cmul_fpoly(FPoly, float, FPoly);
void subst_fpoly(FPoly, FPoly);
void set0_fpoly(FPoly);
long int max_abscoef_fpoly(FPoly);
long int num_nonzero_fpoly(FPoly);
void diff_fpoly(FPoly);
float eval_fpoly(FPoly, float);
float eval_diff_fpoly(FPoly, float);
void ceval_fpoly(FCmplx, FPoly, FCmplx);
void ceval_diff_fpoly(FCmplx, FPoly, FCmplx);
void add_dpoly(DPoly, DPoly, DPoly);
void sub_dpoly(DPoly, DPoly, DPoly);
void mul_dpoly(DPoly, DPoly, DPoly);
void cmul_dpoly(DPoly, double, DPoly);
void subst_dpoly(DPoly, DPoly);
void set0_dpoly(DPoly);
long int max_abscoef_dpoly(DPoly);
long int num_nonzero_dpoly(DPoly);
void diff_dpoly(DPoly);
double eval_dpoly(DPoly, double);
double eval_diff_dpoly(DPoly, double);
void ceval_dpoly(DCmplx, DPoly, DCmplx);
void ceval_diff_dpoly(DCmplx, DPoly, DCmplx);
#ifdef USE_GMP
MPFPoly init_mpfpoly(long int);
MPFPoly init2_mpfpoly(long int, unsigned long);
void free_mpfpoly(MPFPoly);
mpf_ptr get_mpfpoly_i(MPFPoly, long int);
void set_mpfpoly_i(MPFPoly, long int, mpf_t);
void set_mpfpoly_i_d(MPFPoly, long int, double);
void set_mpfpoly_i_si(MPFPoly, long int, long);
void set_mpfpoly_i_ui(MPFPoly, long int, unsigned long);
void set_mpfpoly_i_str(MPFPoly, long int, const char *, int);
void print_mpfpoly(MPFPoly);
void print_fdmpfpoly(FPoly, DPoly, MPFPoly);
void add_mpfpoly(MPFPoly, MPFPoly, MPFPoly);
void sub_mpfpoly(MPFPoly, MPFPoly, MPFPoly);
void mul_mpfpoly(MPFPoly, MPFPoly, MPFPoly);
void cmul_mpfpoly(MPFPoly, mpf_t, MPFPoly);
void subst_mpfpoly(MPFPoly, MPFPoly);
void set0_mpfpoly(MPFPoly);
long int max_abscoef_mpfpoly(MPFPoly);
long int num_nonzero_mpfpoly(MPFPoly);
void diff_mpfpoly(MPFPoly);
void eval_mpfpoly(mpf_t, MPFPoly, mpf_t);
void eval_diff_mpfpoly(mpf_t, MPFPoly, mpf_t);
void ceval_mpfpoly(MPFCmplx, MPFPoly, MPFCmplx);
void ceval_diff_mpfpoly(MPFCmplx, MPFPoly, MPFCmplx);
#endif

/*************************************************/
/* complex.c: Complex arithmetic                 */
/*************************************************/
FCmplx init_fcmplx(void);
void free_fcmplx(FCmplx);
float get_real_fcmplx(FCmplx);
float get_image_fcmplx(FCmplx);
void set_real_fcmplx(FCmplx, float);
void set_image_fcmplx(FCmplx, float);
void subst_fcmplx(FCmplx, FCmplx);
void set0_fcmplx(FCmplx);
void add_fcmplx(FCmplx, FCmplx, FCmplx);
void add_fcmplx_f(FCmplx, FCmplx, float);
void add2_fcmplx(FCmplx, FCmplx);
void sub_fcmplx(FCmplx, FCmplx, FCmplx);
void conj_fcmplx(FCmplx, FCmplx);
void sign_fcmplx(FCmplx, FCmplx);
void sign2_fcmplx(FCmplx);
float abs_fcmplx(FCmplx);
void mul_fcmplx(FCmplx, FCmplx, FCmplx);
void mul_fcmplx_f(FCmplx, FCmplx, float);
void mul2_fcmplx(FCmplx, FCmplx);
void div_fcmplx(FCmplx, FCmplx, FCmplx);
void iexp_fcmplx(FCmplx, float);
void print_fcmplx(FCmplx);

DCmplx init_dcmplx(void);
void free_dcmplx(DCmplx);
double get_real_dcmplx(DCmplx);
double get_image_dcmplx(DCmplx);
void set_real_dcmplx(DCmplx, double);
void set_image_dcmplx(DCmplx, double);
void subst_dcmplx(DCmplx, DCmplx);
void set0_dcmplx(DCmplx);
void add_dcmplx(DCmplx, DCmplx, DCmplx);
void add_dcmplx_d(DCmplx, DCmplx, double);
void add2_dcmplx(DCmplx, DCmplx);
void sub_dcmplx(DCmplx, DCmplx, DCmplx);
void conj_dcmplx(DCmplx, DCmplx);
void sign_dcmplx(DCmplx, DCmplx);
void sign2_dcmplx(DCmplx);
double abs_dcmplx(DCmplx);
void mul_dcmplx(DCmplx, DCmplx, DCmplx);
void mul_dcmplx_d(DCmplx, DCmplx, double);
void mul2_dcmplx(DCmplx, DCmplx);
void div_dcmplx(DCmplx, DCmplx, DCmplx);
void iexp_dcmplx(DCmplx, double);
void print_dcmplx(DCmplx);

#ifdef USE_GMP
MPFCmplx init_mpfcmplx(void);
MPFCmplx init2_mpfcmplx(unsigned long int);
void free_mpfcmplx(MPFCmplx);
void get_real_mpfcmplx(mpf_t, MPFCmplx);
void get_image_mpfcmplx(mpf_t, MPFCmplx);
void set_real_mpfcmplx(MPFCmplx, mpf_t);
void set_image_mpfcmplx(MPFCmplx, mpf_t);
void set_real_mpfcmplx_ui(MPFCmplx, unsigned long int);
void set_image_mpfcmplx_ui(MPFCmplx, unsigned long int);
void subst_mpfcmplx(MPFCmplx, MPFCmplx);
void set0_mpfcmplx(MPFCmplx);
void add_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void add_mpfcmplx_mpf(MPFCmplx, MPFCmplx, mpf_t);
void add2_mpfcmplx(MPFCmplx, MPFCmplx);
void sub_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void conj_mpfcmplx(MPFCmplx, MPFCmplx);
void sign_mpfcmplx(MPFCmplx, MPFCmplx);
void sign2_mpfcmplx(MPFCmplx);
void abs_mpfcmplx(mpf_t, MPFCmplx);
void mul_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void mul_mpfcmplx_ui(MPFCmplx, MPFCmplx, unsigned long int);
void mul_mpfcmplx_mpf(MPFCmplx, MPFCmplx, mpf_t);
void mul2_mpfcmplx(MPFCmplx, MPFCmplx);
void div_mpfcmplx(MPFCmplx, MPFCmplx, MPFCmplx);
void iexp_mpfcmplx(MPFCmplx, mpf_t);
void print_mpfcmplx(MPFCmplx);

/* set unsigned long */
void set_mpfcmplx_ui_ui(MPFCmplx, unsigned long, unsigned long);

/* set strings as complex number */
void set_mpfcmplx_str_str(MPFCmplx, const char *, long, const char *, long);

/* set double precision real part */
void set_real_mpfcmplx_d(MPFCmplx, double);

/* set double precision imaginary part */
void set_image_mpfcmplx_d(MPFCmplx, double);

void set_mpfcmplx_d(MPFCmplx, double complex);

unsigned long int get_prec_mpfcmplx(MPFCmplx);
mpf_ptr getp_real_mpfcmplx(MPFCmplx);
mpf_ptr getp_image_mpfcmplx(MPFCmplx);

/* c := a / b */
void div_mpfcmplx_d(MPFCmplx, MPFCmplx, double complex);

/* c := 1 / a */
void inv_mpfcmplx(MPFCmplx, MPFCmplx);

/* c := -a */
void neg_mpfcmplx(MPFCmplx, MPFCmplx);
#endif


/*************************************************/
/* integral.c: Numerical Integral                */
/*************************************************/
float ftrapezoidal_fs(float, float, float(* func)(float), long int);
double dtrapezoidal_fs(double, double, double(* func)(double), long int);
double dmtrapezoidal_fs(double, double, double(* func)(double), long int);
#ifdef USE_GMP
void mpf_trapezoidal_fs(mpf_t, mpf_t, mpf_t, void(* func)(mpf_t, mpf_t), long int);
void mpf_mtrapezoidal_fs(mpf_t, mpf_t, mpf_t, void(* func)(mpf_t, mpf_t), long int);
#endif

/*************************************************/
/* dka.c: DKA Methods                            */
/*************************************************/
float fdka_center(FPoly);
float fdka_radius(FPoly);
void fdka_init(CFArray, FPoly);
long int fdka(CFArray, CFArray, FPoly, long int, float, float);

double ddka_center(DPoly);
double ddka_radius(DPoly);
void ddka_init(CDArray, DPoly);
long int ddka(CDArray, CDArray, DPoly, long int, double, double);

#ifdef USE_GMP
void mpf_dka_center(mpf_t, MPFPoly);
void mpf_dka_radius(mpf_t, MPFPoly);
void mpf_dka_init(CMPFArray, MPFPoly);
long int mpf_dka(CMPFArray, CMPFArray, MPFPoly, long int, mpf_t, mpf_t);
#endif

/*****************************************************/
/* iterative.c: Iterative Methods for Linear Systems */
/*****************************************************/
void get_residual_fvector(FVector, FVector, FMatrix, FVector);
void fjacobi(FVector, FMatrix, FVector, float, float, long int);
void fgs(FVector, FMatrix, FVector, float, float, long int);
void fsor(FVector, FMatrix, FVector, float, float, float, long int);

void get_residual_dvector(DVector, DVector, DMatrix, DVector);
void djacobi(DVector, DMatrix, DVector, double, double, long int);
void dgs(DVector, DMatrix, DVector, double, double, long int);
void dsor(DVector, DMatrix, DVector, double, double, double, long int);

#ifdef USE_GMP
void mpf_jacobi(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long int);
void mpf_gs(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long int);
void mpf_sor(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, mpf_t, long int);
#endif

/************************************************/
/* qr.c: Gram-Schmidt, Modified Gram-Schmidt, QR*/
/************************************************/
void dgram_schmidt(DMatrix, DMatrix, DMatrix);
void dmgram_schmidt(DMatrix, DMatrix, DMatrix);
void dqr(DMatrix, long int);

#ifdef USE_GMP
void mpf_gram_schmidt(MPFMatrix, MPFMatrix, MPFMatrix);
void mpf_mgram_schmidt(MPFMatrix, MPFMatrix, MPFMatrix);
void mpf_qr(MPFMatrix, long int);
#endif

/************************************************/
/* gtestmat.c: Get Test Matrices                */
/************************************************/
void hilbert_dmatrix(DMatrix, long int);
void lotkin_dmatrix(DMatrix, long int);
void frank_dmatrix(DMatrix, long int);
void tridiag_dmatrix(DMatrix, DVector, DVector, DVector, long int);
void int_sym_rand_dmatrix(DMatrix, long int, long int, long int);
void int_unsym_rand_dmatrix(DMatrix, long int, long int, long int);
void diag_dmatrix(DMatrix, DVector, long int);
void toeplitz_dmatrix(DMatrix, double, long int);

#ifdef USE_GMP
void hilbert_mpfmatrix(MPFMatrix, long int);
void lotkin_mpfmatrix(MPFMatrix, long int);
void frank_mpfmatrix(MPFMatrix, long int);
void tridiag_mpfmatrix(MPFMatrix, MPFVector, MPFVector, MPFVector, long int);
void int_sym_rand_mpfmatrix(MPFMatrix, long int, long int, long int);
void int_unsym_rand_mpfmatrix(MPFMatrix, long int, long int, long int);
void diag_mpfmatrix(MPFMatrix, MPFVector, long int);
void toeplitz_mpfmatrix(MPFMatrix, mpf_t, long int);
#endif

/*************************************************/
/* rational.c: Functions for Vector Types        */
/*************************************************/

#ifdef USE_GMP
void mpq_get_f(mpf_t, mpq_t);

MPQVector init_mpqvector(long int dimension);
void free_mpqvector(MPQVector vec);

mpq_ptr get_mpqvector_i(MPQVector vec, long int index);
void set_mpqvector_i(MPQVector vec, long int index, mpq_t val);
void set_mpqvector_i_z(MPQVector vec, long int index, mpz_t val);
void set_mpqvector_i_str_b(MPQVector vec, long int index, const char *str, int base);
void set_mpqvector_i_str(MPQVector vec, long int index, const char *str);
void set_mpqvector_i_ui(MPQVector vec, long int index, unsigned long val_num, unsigned long val_den);
void set_mpqvector_i_si(MPQVector vec, long int index, long val_num, unsigned long val_den);

void print_mpqvector(MPQVector vec);

void ip_mpqvector(mpq_t ret, MPQVector a, MPQVector b);
#endif

/*************************************************/
/* krylov.c: Krylov Subspace Methods             */
/*************************************************/
long int DBiCG(DVector, DMatrix, DVector, double, double, long);
long int DCGS(DVector, DMatrix, DVector, double, double, long);
long int DBiCGSTAB(DVector, DMatrix, DVector, double, double, long);
long int DGPBiCG(DVector, DMatrix, DVector, double, double, long);
#ifdef USE_GMP
long int MPFBiCG(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
long int MPFCGS(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
long int MPFBiCGSTAB(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
long int MPFGPBiCG(MPFVector, MPFMatrix, MPFVector, mpf_t, mpf_t, long);
#endif

/*************************************************/
/* diff.c: Numerical Differentiations            */
/*************************************************/
double drelerr(double, double);
double dcentral_diff(double, double (*func)(double), double);
double dcentral_diff15(double, double (*func)(double), double);
double dcentral_diff17(double, double (*func)(double), double);
double dfnmdiff(double, double (*func)(double), double, double, double, long int, long int *);
void djfnmdiff(DMatrix, DVector, void (*func)(DVector, double, DVector), double, double, double, long int);
#ifdef USE_GMP
void mpfrelerr(mpf_t, mpf_t, mpf_t);
void mpf_central_diff(mpf_t, mpf_t, void (*func)(mpf_t, mpf_t), mpf_t);
void mpf_central_diff15(mpf_t, mpf_t, void (*func)(mpf_t, mpf_t), mpf_t);
void mpf_central_diff17(mpf_t, mpf_t, void (*func)(mpf_t, mpf_t), mpf_t);
void mpffnmdiff(mpf_t, mpf_t, void (*func)(mpf_t, mpf_t), mpf_t, mpf_t, mpf_t, long int, long int *);
void mpf_jfnmdiff(MPFMatrix, MPFVector, void (*func)(MPFVector, mpf_t, MPFVector), mpf_t, mpf_t, mpf_t, long int);
#endif

/*************************************************/
/* fread_write.c: Read and Write files           */
/*************************************************/
void fread_dmatrix(FILE *, DMatrix);
void fread_dmatrix_fname(const char *, DMatrix);
void fwrite_dmatrix(FILE *, DMatrix);
void fwrite_dmatrix_fname(const char *, DMatrix);
void fread_dpolycoef(FILE *, DPoly p, long int);
void fread_dpolycoef_fname(const char *, DPoly p, long int);
void fread_dvector(FILE *, DVector);
void fread_dvector_fname(const char *, DVector);
void fwrite_dvector(FILE *, DVector);
void fwrite_dvector_fname(const char *, DVector);
#ifdef USE_GMP
void fread_mpfmatrix(FILE *, MPFMatrix);
void fread_mpfmatrix_fname(const char *, MPFMatrix);
void fwrite_mpfmatrix(FILE *, MPFMatrix);
void fwrite_mpfmatrix_fname(const char *, MPFMatrix);
void fread_mpolycoef(FILE *, MPFPoly, long int);
void fread_mpolycoef_fname(const char *, MPFPoly, long int);
void fread_mpfvector(FILE *, MPFVector);
void fread_mpfvector_fname(const char *, MPFVector);
void fwrite_mpfvector(FILE *, MPFVector);
void fwrite_mpfvector_fname(const char *, MPFVector);
#endif

/*************************************************/
/* mharmonic.c: Extrapolation for Integration    */
/*************************************************/
unsigned long ex_nim_den(long int);
double ex_harmonic_den(long int istage, long int jstage);
double dmharmonic_integral_1step(double x_start, double x_end, double (* func)(double), long int num_stage, double reps, double aeps, long int *conv_flag);
double dmharmonic_integral(double x_start, double x_end, double (* func)(double), long int max_num_stage);
double dmharmonic_backward_integral(double x_start, double x_end, double (* func)(double), long int max_num_stage);
double dmromberg_integral_1step(double x_start, double x_end, double (* func)(double), long int num_stage, double reps, double aeps, long int *conv_flag);
double dmromberg_integral(double x_start, double x_end, double (* func)(double), long int max_num_stage);
#ifdef USE_GMP
void mpf_ex_harmonic_den(mpf_t ret, long int istage, long int jstage);
void mpf_ex_harmonic1_den(mpf_t ret, long int istage, long int jstage);
void mpf_ex_harmonic2_den(mpf_t ret, long int istage, long int jstage);
void mpf_ex_harmonic3_den(mpf_t ret, long int istage, long int jstage);
void mpf_mharmonic_integral_1step(mpf_t ret, mpf_t x_start, mpf_t x_end, void (* func)(mpf_t, mpf_t), long int num_stage, mpf_t reps, mpf_t aeps, long int *conv_flag);
void mpf_mharmonic_integral(mpf_t ret, mpf_t x_start, mpf_t x_end, void (* func)(mpf_t, mpf_t), long int max_num_stage);
void mpf_mromberg_integral_1step(mpf_t ret, mpf_t x_start, mpf_t x_end, void (* func)(mpf_t, mpf_t), long int num_stage, mpf_t reps, mpf_t aeps, long int *conv_flag);
void mpf_mromberg_integral(mpf_t ret, mpf_t x_start, mpf_t x_end, void (* func)(mpf_t, mpf_t), long int max_num_stage);
#endif

/*************************************************************/
/* algebraic_eq.c: Algebraic Solvers for Algebraic Equations */
/*************************************************************/
int dquadratic_eq(double[2], double[2], double[3]);
#ifdef USE_GMP
int mpfquadratic_eq(mpf_t[2], mpf_t[2], mpf_t[3]);
#endif

#if defined (__cplusplus)
}
#endif

/**********************************************/
/* clinear.c: Complex linear computation      */
/**********************************************/
// CDVector: double precision complex vector
// based on double complex defined in C99
typedef struct{
	double complex *element;
	long int dim;
} cdvector;

typedef cdvector *CDVector;

// CDMatrix: double precision complex matrix
// based on double complex defined in C99
typedef struct{
	double complex *element;
	long int row_dim, col_dim;
} cdmatrix;

typedef cdmatrix *CDMatrix;

#ifdef USE_GMP
// CMPFVector: multiple precision complex vector
typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int dim;
} cmpfvector;

typedef cmpfvector *CMPFVector;

// CMPFMatrix: multiple precision complex matrix
typedef struct{
	unsigned long int prec;
	mpfcmplx *element;
	long int row_dim, col_dim;
} cmpfmatrix;

typedef cmpfmatrix *CMPFMatrix;
#endif

CDVector init_cdvector(long int);
void free_cdvector(CDVector);
double complex get_cdvector_i(CDVector, long int);
void set_cdvector_i(CDVector vec, long int, double complex);
void print_cdvector(CDVector);
void print_cdmatrix(CDMatrix);
void add_cdvector(CDVector, CDVector, CDVector);
void add2_cdvector(CDVector, CDVector);
void sub_cdvector(CDVector, CDVector, CDVector);
void sub2_cdvector(CDVector, CDVector);
void cmul_cdvector(CDVector, double complex, CDVector);
void cmul2_cdvector(CDVector, double complex);
void add_cmul_cdvector(CDVector, CDVector, double complex, CDVector);
double complex ip_cdvector(CDVector, CDVector);
double norm1_cdvector(CDVector);
double norm2_cdvector(CDVector);
double normi_cdvector(CDVector);
void subst_cdvector(CDVector, CDVector);
void subst_cdvector_dvec(CDVector, DVector);
void subst_dvector_real_cdvec(DVector, CDVector);
void subst_dvector_image_cdvec(DVector, CDVector);
void set0_cdvector(CDVector);
void copy_cdvector_ij(CDVector, long int, long int, CDVector, long int, long int);
CDMatrix init_cdmatrix(long int, long int);
void free_cdmatrix(CDMatrix);
double complex get_cdmatrix_ij(CDMatrix, long int, long int);
double complex get_cdmatrix_ij(CDMatrix, long int, long int);
void print_cdmatrix(CDMatrix);
double normf_cdmatrix(CDMatrix);
double normi_cdmatrix(CDMatrix);
double norm1_cdmatrix(CDMatrix);
void add_cdmatrix(CDMatrix, CDMatrix, CDMatrix);
void sub_cdmatrix(CDMatrix, CDMatrix, CDMatrix);
void cmul_cdmatrix(CDMatrix, double complex, CDMatrix);
void mul_cdmatrix(CDMatrix, CDMatrix, CDMatrix);
void transpose_cdmatrix(CDMatrix, CDMatrix);
void subst_cdmatrix(CDMatrix, CDMatrix);
void subst_cdmatrix_dmat(CDMatrix, DMatrix);
void set0_cdmatrix(CDMatrix);
void setI_cdmatrix(CDMatrix);
void mul_cdmatrix_cdvec(CDVector, CDMatrix, CDVector);
void mul_cdmatrixt_cdvec(CDVector, CDMatrix, CDVector);
void inv_cdmatrix(CDMatrix);

#ifdef USE_GMP
CMPFVector init_cmpfvector(long int);
CMPFVector init2_cmpfvector(long int, unsigned long int);
void free_cmpfvector(CMPFVector);
void print_cmpfmatrix(CMPFMatrix);
MPFCmplx get_cmpfvector_i(CMPFVector, long int);
MPFCmplx get_cmpfvector_i(CMPFVector, long int);
void set_cmpfvector_i(CMPFVector, long int, MPFCmplx);
void set_cmpfvector_i_d(CMPFVector, long int, double complex);
void set_cmpfvector_i_str(CMPFVector, long int, const char *, const char *, int);
void set_cmpfvector_i_ui(CMPFVector, long int, unsigned long);
unsigned long int prec_cmpfvector(CMPFVector);
unsigned long int get_prec_cmpfvector(CMPFVector);
unsigned long int minprec_cmpfvector(CMPFVector);
unsigned long int maxprec_cmpfvector(CMPFVector);
void print_cmpfvector(CMPFVector);
void add_cmpfvector(CMPFVector, CMPFVector, CMPFVector);
void add2_cmpfvector(CMPFVector, CMPFVector);
void sub_cmpfvector(CMPFVector, CMPFVector, CMPFVector);
void sub2_cmpfvector(CMPFVector, CMPFVector);
void cmul_cmpfvector(CMPFVector, MPFCmplx, CMPFVector);
void cmul_cmpfvector_mpfvec(CMPFVector, MPFCmplx, MPFVector);
void cmul2_cmpfvector(CMPFVector, MPFCmplx);
void add_cmul_cmpfvector(CMPFVector, CMPFVector, MPFCmplx, CMPFVector);
void ip_cmpfvector(MPFCmplx, CMPFVector, CMPFVector);
void norm1_cmpfvector(mpf_t, CMPFVector);
void subst_cmpfvector(CMPFVector, CMPFVector);
void norm2_cmpfvector(mpf_t, CMPFVector);
void normi_cmpfvector(mpf_t, CMPFVector);
void subst_cmpfvector_mpfvec(CMPFVector, MPFVector);
void subst_mpfvector_image_cmpfvec(MPFVector, CMPFVector);
void subst_mpfvector_real_cmpfvec(MPFVector, CMPFVector);
void set0_cmpfvector(CMPFVector);
void copy_cmpfvector_ij(CMPFVector, long int, long int, CMPFVector, long int, long int);
CMPFMatrix init_cmpfmatrix(long int, long int);
CMPFMatrix init2_cmpfmatrix(long int, long int, unsigned long);
void free_cmpfmatrix(CMPFMatrix);
MPFCmplx get_cmpfmatrix_ij(CMPFMatrix, long int, long int);
void set_cmpfmatrix_ij(CMPFMatrix, long int, long int, MPFCmplx);
void set_cmpfmatrix_ij_d(CMPFMatrix, long int, long int, double complex);
void set_cmpfmatrix_ij_str(CMPFMatrix, long int, long int, const char *, const char *, int);
void set_cmpfmatrix_ij_ui(CMPFMatrix, long int, long int, unsigned long );
unsigned long int maxprec_cmpfmatrix(CMPFMatrix);
unsigned long int get_prec_cmpfmatrix(CMPFMatrix);
unsigned long int prec_cmpfmatrix(CMPFMatrix);
unsigned long int minprec_cmpfmatrix(CMPFMatrix);
void print_cmpfmatrix(CMPFMatrix);
void normf_cmpfmatrix(mpf_t, CMPFMatrix);
void normi_cmpfmatrix(mpf_t, CMPFMatrix);
void norm1_cmpfmatrix(mpf_t, CMPFMatrix);
void add_cmpfmatrix(CMPFMatrix, CMPFMatrix, CMPFMatrix);
void sub_cmpfmatrix(CMPFMatrix, CMPFMatrix, CMPFMatrix);
void cmul_cmpfmatrix(CMPFMatrix, MPFCmplx, CMPFMatrix);
void mul_cmpfmatrix(CMPFMatrix, CMPFMatrix, CMPFMatrix);
void transpose_cmpfmatrix(CMPFMatrix, CMPFMatrix);
void subst_cmpfmatrix(CMPFMatrix, CMPFMatrix);
void subst_cmpfmatrix_mpfmat(CMPFMatrix, MPFMatrix);
void set0_cmpfmatrix(CMPFMatrix);
void setI_cmpfmatrix(CMPFMatrix);
void mul_cmpfmatrix_cmpfvec(CMPFVector, CMPFMatrix, CMPFVector);
void mul_cmpfmatrixt_cmpfvec(CMPFVector, CMPFMatrix, CMPFVector);
void inv_cmpfmatrix(CMPFMatrix);
#endif

/**********************************************/
/* clu.c : LU decomposition of complex matrix */
/**********************************************/
int CDLUdecomp(CDMatrix);
int SolveCDLS(CDVector, CDMatrix, CDVector);
int CDLUdecompP(CDMatrix, long int[]);
int SolveCDLSP(CDVector, CDMatrix, CDVector, long int[]);
int CDLUdecompC(CDMatrix, long int[], long int[]);
int SolveCDLSC(CDVector, CDMatrix, CDVector, long int[], long int[]);

#ifdef USE_GMP
int CMPFLUdecomp(CMPFMatrix);
int SolveCMPFLS(CMPFVector, CMPFMatrix, CMPFVector);
int CMPFLUdecompP(CMPFMatrix, long int[]);
int SolveCMPFLSP(CMPFVector, CMPFMatrix, CMPFVector, long int[]);
int CMPFLUdecompC(CMPFMatrix, long int[], long int[]);
int SolveCMPFLSC(CMPFVector, CMPFMatrix, CMPFVector, long int[], long int[]);
#endif

/**********************************************/
/* bmatrix.c : Bandmatrix                     */
/**********************************************/
FBMatrix init_fbmatrix(long int, long int, long int);
void free_fbmatrix(FBMatrix);
float get_fbmatrix_ij(FBMatrix, long int, long int);
void set_fbmatrix_ij(FBMatrix, long int, long int, float);
int mul_fbmatrix_fvec(FVector, FBMatrix, FVector);
int mul_fbmatrixt_fvec(FVector, FBMatrix, FVector);
int FBLUdecomp(FBMatrix);
int SolveFBLS(FVector, FBMatrix, FVector);
void print_fbmatrix(FBMatrix);
void set0_fbmatrix(FBMatrix);
void setI_fbmatrix(FBMatrix);
void add_fbmatrix(FBMatrix, FBMatrix, FBMatrix);
void sub_fbmatrix(FBMatrix, FBMatrix, FBMatrix);
void sub_fmatrix_fbmat_fmat(FMatrix, FBMatrix, FMatrix);
void cmul_fbmatrix(FBMatrix, float, FBMatrix);
void mul_fmatrix_fbmat_fbmat(FMatrix, FBMatrix, FBMatrix);
void mul_fmatrix_fmat_fbmat(FMatrix, FMatrix, FBMatrix);
void mul_fmatrix_fbmat_fmat(FMatrix, FBMatrix, FMatrix);
void subst_fbmatrix(FBMatrix, FBMatrix);
void subst_fmatrix_fbmat(FMatrix, FBMatrix);

DBMatrix init_dbmatrix(long int, long int, long int);
void free_dbmatrix(DBMatrix);
double get_dbmatrix_ij(DBMatrix, long int, long int);
void set_dbmatrix_ij(DBMatrix, long int, long int, double);
int mul_dbmatrix_dvec(DVector, DBMatrix, DVector);
int mul_dbmatrixt_dvec(DVector, DBMatrix, DVector);
int DBLUdecomp(DBMatrix);
int SolveDBLS(DVector, DBMatrix, DVector);
void print_dbmatrix(DBMatrix);
void set0_dbmatrix(DBMatrix);
void setI_dbmatrix(DBMatrix);
void add_dbmatrix(DBMatrix, DBMatrix, DBMatrix);
void sub_dbmatrix(DBMatrix, DBMatrix, DBMatrix);
void sub_dmatrix_dbmat_dmat(DMatrix, DBMatrix, DMatrix);
void cmul_dbmatrix(DBMatrix, double, DBMatrix);
void mul_dmatrix_dbmat_dbmat(DMatrix, DBMatrix, DBMatrix);
void mul_dmatrix_dmat_dbmat(DMatrix, DMatrix, DBMatrix);
void mul_dmatrix_dbmat_dmat(DMatrix, DBMatrix, DMatrix);
void subst_dbmatrix(DBMatrix, DBMatrix);
void subst_fbmatrix_dbmat(FBMatrix, DBMatrix);
void subst_dbmatrix_fbmat(DBMatrix, FBMatrix);
void subst_dmatrix_dbmat(DMatrix, DBMatrix);

#ifdef USE_GMP
MPFBMatrix init_mpfbmatrix(long int, long int, long int);
MPFBMatrix init2_mpfbmatrix(long int, long int, long int, unsigned long);
void free_mpfbmatrix(MPFBMatrix);
mpf_ptr get_mpfbmatrix_ij(MPFBMatrix, long int, long int);
void set_mpfbmatrix_ij(MPFBMatrix, long int, long int, mpf_t);
void set_mpfbmatrix_ij_d(MPFBMatrix, long int, long int, double);
void set_mpfbmatrix_ij_ui(MPFBMatrix, long int, long int, unsigned long);
int mul_mpfbmatrix_mpfvec(MPFVector, MPFBMatrix, MPFVector);
int mul_mpfbmatrixt_mpfvec(MPFVector, MPFBMatrix, MPFVector);
int MPFBLUdecomp(MPFBMatrix);
int SolveMPFBLS(MPFVector, MPFBMatrix, MPFVector);
void print_mpfbmatrix(MPFBMatrix);
void set0_mpfbmatrix(MPFBMatrix);
void setI_mpfbmatrix(MPFBMatrix);
void add_mpfbmatrix(MPFBMatrix, MPFBMatrix, MPFBMatrix);
void sub_mpfbmatrix(MPFBMatrix, MPFBMatrix, MPFBMatrix);
void sub_mpfmatrix_mpfbmat_mpfmat(MPFMatrix, MPFBMatrix, MPFMatrix);
void cmul_mpfbmatrix(MPFBMatrix, mpf_t, MPFBMatrix);
void mul_mpfmatrix_mpfbmat_mpfbmat(MPFMatrix, MPFBMatrix, MPFBMatrix);
void mul_mpfmatrix_mpfmat_mpfbmat(MPFMatrix, MPFMatrix, MPFBMatrix);
void mul_mpfmatrix_mpfbmat_mpfmat(MPFMatrix, MPFBMatrix, MPFMatrix);
void subst_mpfbmatrix(MPFBMatrix, MPFBMatrix);
void subst_dbmatrix_mpfbmat(DBMatrix, MPFBMatrix);
void subst_mpfbmatrix_dbmat(MPFBMatrix, DBMatrix);
void subst_mpfmatrix_mpfbmat(MPFMatrix, MPFBMatrix);
#endif

#endif /* End of __BNC_H */
