/********************************************************************************/
/* Test program for taylor.c                                                    */
/* Copyright (C) 2000-2011 Tomonori Kouya                                       */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
#include <stdio.h>
#include <math.h>

#include "bncode.h"

/* dy/dx = f(x, y) */
void df(DTaylor y[], double x0, DTaylor y0[], long int dim)
{
	long int i;

	/* y[i] := y0[i] */
	for(i = 0; i < dim; i++)
		copy_dtaylor(y[i], y0[i]);
}

double ans(double x)
{
	return exp(x);
}

/* dy/dx = -y */
void df1(DTaylor y[], double x0, DTaylor y0[], long int dim)
{
	long int i;

	/* y[i] := -y0[i]  */
	for(i = 0; i < dim; i++)
	{
		neg_dtaylor(y[i], y0[i]);
	}
}

/* dy/dx = -y */
void mpf_df1(MPFTaylor y[], mpf_t x0, MPFTaylor y0[], long int dim)
{
	long int i;

	/* y[i] := -y0[i]  */
	for(i = 0; i < dim; i++)
	{
		neg_mpftaylor(y[i], y0[i]);
	}
}

void df1_1(DVector y, double x0, DVector y0)
{
	long int i;

	for(i = 0; i < y->dim; i++)
		sdvi(y, i, -gdvi(y0, i));
}

void mpf_df1_1(MPFVector y, mpf_t x0, MPFVector y0)
{
	long int i;
	mpf_t tmp;

	mpf_init2(tmp, y->prec);
	for(i = 0; i < y->dim; i++)
	{
		mpf_set(tmp, gmpfvi(y0, i));
		mpf_neg(tmp, tmp);
		smpfvi(y, i, tmp);
	}
}

double ans1(double x)
{
	return exp(-x);
}

#define ALPHA 0.999
#define ALPHA_STR "0.999"

/* dy/dx = alpha * y^2 * cos(x) */
void df2(DTaylor y[], double x0, DTaylor y0[], long int dim)
{
	static int flag = 0;
	long int i;
	DTaylor tcos, tcos2;

	tcos = init_dtaylor();
	tcos2 = init_dtaylor();

	set_cos2_dtaylor(tcos, x0);
//	set_cos_dtaylor(tcos2);
//	shift_dtaylor(tcos, tcos2, x0);

	/* y[i] := alpha * y[i]^2 * cos(x0) */
	for(i = 0; i < dim; i++)
	{
		mul_dtaylor(y[i], y0[i], y0[i]);
		mul_dtaylor(y[i], tcos, y[i]);
		smul_dtaylor(y[i], ALPHA, y[i]);
	}

	free_dtaylor(tcos);
	free_dtaylor(tcos2);
}

/* dy/dx = alpha * y^2 * cos(x) */
void mpf_df2(MPFTaylor y[], mpf_t x0, MPFTaylor y0[], long int dim)
{
	static int flag = 0;
	long int i;
	MPFTaylor tcos, tcos2;
	mpf_t tmp;

	tcos = init_mpftaylor();
	tcos2 = init_mpftaylor();
	mpf_init2(tmp, y[0]->prec);

//	set_cos2_mpftaylor(tcos, x0);
	set_cos_mpftaylor(tcos2);
	shift_mpftaylor(tcos, tcos2, x0);

	/* y[i] := alpha * y[i]^2 * cos(x0) */
	for(i = 0; i < dim; i++)
	{
		mul_mpftaylor(y[i], y0[i], y0[i]);
		mul_mpftaylor(y[i], tcos, y[i]);
		mpf_set_str(tmp, ALPHA_STR, 10);
		smul_mpftaylor(y[i], tmp, y[i]);
	}

	mpf_clear(tmp);
	free_mpftaylor(tcos);
	free_mpftaylor(tcos2);
}

/* dy/dx = alpha * y^2 * cos(x) */
void df2_1(DVector y, double x0, DVector y0)
{
	static int flag = 0;
	long int i;

	/* y[i] := alpha * y[i]^2 * cos(x0) */
	for(i = 0; i < y0->dim; i++)
	{
		sdvi(y, i, ALPHA * gdvi(y0, i) * gdvi(y0, i) * cos(x0));
	}
}

/* dy/dx = alpha * y^2 * cos(x) */
void mpf_df2_1(MPFVector y, mpf_t x0, MPFVector y0)
{
	static int flag = 0;
	long int i;
	mpf_t tmp, tmp1, tmp2;

	mpf_init2(tmp, y->prec);
	mpf_init2(tmp1, y->prec);
	mpf_init2(tmp2, y->prec);

	/* y[i] := alpha * y[i]^2 * cos(x0) */
	for(i = 0; i < y0->dim; i++)
	{
		mpf_set(tmp, gmpfvi(y0, i));
		mpf_mul(tmp, tmp, tmp);
		mpf_set_str(tmp1, ALPHA_STR, 10);
		mpf_cos(tmp2, x0);
		mpf_mul(tmp, tmp, tmp1);
		mpf_mul(tmp, tmp, tmp2);
		smpfvi(y, i, tmp);
	}

	mpf_clear(tmp);
	mpf_clear(tmp1);
	mpf_clear(tmp2);
}

double ans2(double x)
{
	return 1.0/(1-ALPHA * sin(x));
}

/* dy/dx = 100 * (sin(x) - y) */
void df3(DTaylor y[], double x0, DTaylor y0[], long int dim)
{
	static int flag = 0;
	long int i;
	DTaylor tsin, tsin2;

	tsin = init_dtaylor();
	tsin2 = init_dtaylor();

	set_sin2_dtaylor(tsin, x0);
//	set_sin_dtaylor(tsin2);
//	shift_dtaylor(tsin, tsin2, x0);

	/* y[i] := 100 * (sin(x) - y) */
	for(i = 0; i < dim; i++)
	{
		neg_dtaylor(y[i], y0[i]);
		add_dtaylor(y[i], tsin, y[i]);
		smul_dtaylor(y[i], 100.0, y[i]);
	}

	free_dtaylor(tsin);
	free_dtaylor(tsin2);
}

/* dy/dx = 100 * (sin(x) - y) */
void mpf_df3(MPFTaylor y[], mpf_t x0, MPFTaylor y0[], long int dim)
{
	static int flag = 0;
	long int i;
	MPFTaylor tsin, tsin2, ttmp;
	mpf_t tmp;

	tsin = init_mpftaylor();
	tsin2 = init_mpftaylor();
	mpf_init2(tmp, y[0]->prec);

	set_sin2_mpftaylor(tsin, x0);
//	set_sin_mpftaylor(tsin2);
//	shift_mpftaylor(tsin, tsin2, x0);

	/* y[i] := 100 * (sin(x) - y) */
	for(i = 0; i < dim; i++)
	{
		neg_mpftaylor(y[i], y0[i]);
		add_mpftaylor(y[i], tsin, y[i]);
		mpf_set_str(tmp, "100", 10);
		smul_mpftaylor(y[i], tmp, y[i]);
	}

	free_mpftaylor(tsin);
	free_mpftaylor(tsin2);
	mpf_clear(tmp);
}

/* dy/dx = 100 * (sin(x) - y) */
void df3_1(DVector y, double x0, DVector y0)
{
	static int flag = 0;
	long int i;

	for(i = 0; i < y0->dim; i++)
	{
		sdvi(y, i, 100.0 * (sin(x0) - gdvi(y0, i)));
	}
}

/* dy/dx = 100 * (sin(x) - y) */
void mpf_df3_1(MPFVector y, mpf_t x0, MPFVector y0)
{
	static int flag = 0;
	long int i;
	mpf_t tmp, tmp1, tmp2;

	mpf_init2(tmp, y->prec);
	mpf_init2(tmp1, y->prec);
	mpf_init2(tmp2, y->prec);

	for(i = 0; i < y0->dim; i++)
	{
		mpf_set_str(tmp1, "100", 10);
		mpf_sin(tmp2, x0);
		mpf_set(tmp, gmpfvi(y0, i));
		mpf_sub(tmp, tmp2, tmp);
		mpf_mul(tmp, tmp1, tmp);
		smpfvi(y, i, tmp);
	}

	mpf_clear(tmp);
	mpf_clear(tmp1);
	mpf_clear(tmp2);
}

/* y(x) = (sin(x) - 0.01 * cos(x) + 0.01 * exp(-100*x))/1.0001 */
double ans3(double x)
{
	return (sin(x) - 0.01 * cos(x) + 0.01 * exp(-100.0*x)) / 1.0001;
;
}


/* Taylor Expansion Method */
void ivp_dtaylor_1(DTaylor y[], double x0, DTaylor y0[], void (*func)(DTaylor[], double, DTaylor[], long int), double h, long int dim, DTaylor ytmp[], long int max_deg)
{
	long int i, j, k;
	double ret;

	/* init            */
	/* a0 := y0        */
	/* a1 := f(x0, y0) */
	func(ytmp, x0, y0, dim);
	for(i = 0; i < dim; i++)
	{
		*(y[i]->coef + 0) = *(y0[i]->coef + 0);
		*(y[i]->coef + 1) = *(ytmp[i]->coef + 0);
		for(j = 2; j <= y[i]->max_degree; j++)
			*(y[i]->coef + j) = 0.0;
//		set_current_degree(y[i]);
		y[i]->current_degree = 1;
	}
//	print_dtaylor(ytmp[0]);
//	print_dtaylor(y[0]);

	/* y^(k+1) := y^(k) + Integral[x0, x0+h][func(x, y^(k))] */
	for(k = 2; k <= max_deg; k++)
	{
		/* evaluate dy^(k-1)/dx */
		func(ytmp, x0, y, dim);
//	print_dtaylor(ytmp[0]);

//		printf("Degree: %d -> ", k);
		for(i = 0; i < dim; i++)
		{
			*(y[i]->coef + k) = *(ytmp[i]->coef + k - 1) / k;
			set_current_degree(y[i]);
//			print_dtaylor(y[i]);
//			print_dtaylor(ytmp[i]);
		}
	}

	return ;
}
/* Taylor Expansion Method 2 */
void ivp_dtaylor_fstep(double y[], double x0, double y0[], void (*func)(DTaylor[], double, DTaylor[], long int), double h, long int maxtimes, long int dim)
{
	long int i, k, times;
	double ret, xnew;
	DTaylor in_y[100], in_y0[100], in_ytmp[100];

	/* init */
	for(i = 0; i < dim; i++)
	{
		in_y[i] = init_dtaylor();
		in_y0[i] = init_dtaylor_d(y0[i]);
		in_ytmp[i] = init_dtaylor();
	}

	/* Main Loop */
	xnew = x0;
	for(times = 1; times <= maxtimes; times++)
	{
		for(i = 0; i < dim; i++)
		{
			set_const_dtaylor(in_y0[i], y0[i]);
			set_const_dtaylor(in_y[i], 0.0);
			set_const_dtaylor(in_ytmp[i], 0.0);
		}

		/* y(xnew + h) = y(xnew) + y'(xnew) h + ... */
		ivp_dtaylor_1(in_y, xnew, in_y0, func, h, dim, in_ytmp, 16);

/*		printf("\n\n--\nx = %25.17e\n", xnew);
		for(i = 0; i < dim; i++)
			print_dtaylor(in_y[i]);
*/
		/* ret */
		printf("x = %25.17e\n", xnew + h);
		for(i = 0; i < dim; i++)
		{
			y[i] = eval_dtaylor(in_y[i], h);
			printf("%10.7e %5d: %25.17e(%25.17e, %10.7e)\n", xnew + h, i, y[i], ans3(xnew + h), fabs((y[i]-ans3(xnew + h))/ans3(xnew + h)) );
//			printf("%10.7e %5d: %25.17e(%25.17e, %10.7e)\n", xnew + h, i, y[i], ans2(xnew + h), fabs((y[i]-ans2(xnew + h))/ans2(xnew + h)) );
//			printf("%5d: %25.17e(%10.7e)\n", i, y[i], fabs((y[i]-ans1(xnew + h))/ans1(xnew + h)) );
		}

		xnew = x0 + times * h;
		for(i = 0; i < dim; i++)
			y0[i] = y[i];

	}
	/* clear */
	for(i = 0; i < dim; i++)
	{
		free_dtaylor(in_y[i]);
		free_dtaylor(in_y0[i]);
		free_dtaylor(in_ytmp[i]);
	}

	return ;
}

/* Taylor Expansion Method */
void ivp_mpftaylor_1(MPFTaylor y[], mpf_t x0, MPFTaylor y0[], void (*func)(MPFTaylor[], mpf_t, MPFTaylor[], long int), mpf_t h, long int dim, MPFTaylor ytmp[], long int max_deg)
{
	long int i, j, k;
	mpf_t ret;

	/* init            */
	/* a0 := y0        */
	/* a1 := f(x0, y0) */
	func(ytmp, x0, y0, dim);
	for(i = 0; i < dim; i++)
	{
		mpf_set(*(y[i]->coef + 0), *(y0[i]->coef + 0));
		mpf_set(*(y[i]->coef + 1), *(ytmp[i]->coef + 0));
		for(j = 2; j <= y[i]->max_degree; j++)
			mpf_set_ui(*(y[i]->coef + j), 0UL);
//		set_current_degree(y[i]);
		y[i]->current_degree = 1;
	}
//	print_mpftaylor(ytmp[0]);
//	print_mpftaylor(y[0]);

	/* y^(k+1) := y^(k) + Integral[x0, x0+h][func(x, y^(k))] */
	for(k = 2; k <= max_deg; k++)
	{
		/* evaluate dy^(k-1)/dx */
		func(ytmp, x0, y, dim);

//		printf("Degree: %d -> ", k);
		for(i = 0; i < dim; i++)
		{
			mpf_set(*(y[i]->coef + k), *(ytmp[i]->coef + k - 1));
			mpf_div_ui(*(y[i]->coef + k), *(y[i]->coef + k), (unsigned long)k);
			set_current_degree_mpftaylor(y[i]);
//			print_mpftaylor(y[i]);
//			print_mpftaylor(ytmp[i]);
		}
	}

	return ;
}
/* Taylor Expansion Method 2 */
void ivp_mpftaylor_fstep(mpf_t y[], mpf_t x0, mpf_t y0[], void (*func)(MPFTaylor[], mpf_t, MPFTaylor[], long int), mpf_t h, long int maxtimes, long int dim)
{
	long int i, k, times;
	mpf_t ret, xnew;
	MPFTaylor in_y[100], in_y0[100], in_ytmp[100];

	/* init */
	for(i = 0; i < dim; i++)
	{
		in_y[i] = init_mpftaylor();
		in_y0[i] = init_mpftaylor_d(y0[i]);
		in_ytmp[i] = init_mpftaylor();
	}
	mpf_init(xnew);

	/* Main Loop */
	mpf_set(xnew, x0);
	for(times = 1; times <= maxtimes; times++)
	{
		for(i = 0; i < dim; i++)
		{
			set_const_mpftaylor(in_y0[i], y0[i]);
			set_const_mpftaylor_str(in_y[i], "0");
			set_const_mpftaylor_str(in_ytmp[i], "0");
		}

		/* y(xnew + h) = y(xnew) + y'(xnew) h + ... */
		ivp_mpftaylor_1(in_y, xnew, in_y0, func, h, dim, in_ytmp, 32);

/*		printf("\n\n--\nx = %25.17e\n", mpf_get_d(xnew));
		for(i = 0; i < dim; i++)
			print_mpftaylor(in_y[i]);
*/
		/* ret */
		mpf_add(xnew, xnew, h);
		printf("x = %25.17e\n", mpf_get_d(xnew));
		for(i = 0; i < dim; i++)
		{
			eval_mpftaylor(y[i], in_y[i], h);
			printf("%10.7e %5d: %25.17e(%25.17e, %10.7e)\n", mpf_get_d(xnew), i, mpf_get_d(y[i]), ans3(mpf_get_d(xnew )), fabs((mpf_get_d(y[i])-ans3(mpf_get_d(xnew)))/ans3(mpf_get_d(xnew))) );
//			printf("%10.7e %5d: %25.17e(%25.17e, %10.7e)\n", mpf_get_d(xnew), i, mpf_get_d(y[i]), ans2(mpf_get_d(xnew)), fabs((mpf_get_d(y[i])-ans2(mpf_get_d(xnew)))/ans2(mpf_get_d(xnew))) );
//			printf("%5d: %25.17e(%10.7e)\n", i, y[i], fabs((y[i]-ans1(xnew + h))/ans1(xnew + h)) );
		}
		mpf_mul_ui(xnew, h, times);
		mpf_add(xnew, xnew, x0);
		for(i = 0; i < dim; i++)
			mpf_set(y0[i], y[i]);
//		mpf_out_str(stdout, 10, 0, y0[0]); printf("\n");
//		mpf_out_str(stdout, 10, 0, y[0]); printf("\n");

	}
	/* clear */
	for(i = 0; i < dim; i++)
	{
		free_mpftaylor(in_y[i]);
		free_mpftaylor(in_y0[i]);
		free_mpftaylor(in_ytmp[i]);
	}
	mpf_clear(xnew);

	return ;
}

#define DIM 1

main()
{
	DVector y0_v, y_v;
	long int i;
	double y[10], y0[10], x0;

	MPFVector mpf_y0_v, mpf_y_v;
	mpf_t mpf_y[10], mpf_y0[10], mpf_x0, mpf_tmp, mpf_tmp1;

//	goto mpf_ivp;

	set_default_max_degree(16);
	x0 = 0.0;
	y0_v = init_dvector(DIM);
	y_v = init_dvector(DIM);
	for(i = 0; i < DIM; i++)
	{
//		y0[i] = 1.0;
		y0[i] = 0.0; // for ex3 
		sdvi(y0_v, i, y0[i]);
	}

//	goto dex_nim;

	/* y' = y */
	ivp_dtaylor_fstep(y, x0, y0, df3, 0.05, 40, DIM);
//	ivp_dtaylor_fstep(y, x0, y0, df2, 0.1, 10, DIM);
//	ivp_dtaylor_fstep(y, x0, y0, df1, 1.0, 10, DIM);


	for(i = 0; i < DIM; i++)
		printf("%5d: %25.17e %25.17e\n", i, y[i], ans3(2.0));
//		printf("%5d: %25.17e %25.17e\n", i, y[i], ans2(1.0));
//		printf("%5d: %25.17e %25.17e\n", i, y[i], ans1(10.0));
	goto end;

//	derk_fs(stdout, 2.0, y_v, x0, y0_v, 40, df3_1, ERK_BUTCHER1_7_6);
//	derk_fs(stdout, 1.0, y_v, x0, y0_v, 10, df2_1, ERK_BUTCHER1_7_6);
//	derk_fs(stdout, 10.0, y_v, x0, y0_v, 10, df1_1, ERK_BUTCHER1_7_6);

dex_nim:
	dex_nim_fs(stdout, 2.0, y_v, x0, y0_v, 40, df3_1, 0.0, 0.0, 8);
//	dex_nim_fs(stdout, 1.0, y_v, x0, y0_v, 40, df2_1, 0.0, 0.0, 8);
//	dex_nim_fs(stdout, 10.0, y_v, x0, y0_v, 40, df1_1, 0.0, 0.0, 8);

	free_dvector(y0_v);
	free_dvector(y_v);

	goto end;

mpf_ivp:
	set_default_max_degree_mpftaylor(32);
	set_bnc_default_prec(8192);
	mpf_init_set_ui(mpf_x0, 0UL);
	mpf_init(mpf_tmp);
	mpf_init(mpf_tmp1);
	mpf_y0_v = init_mpfvector(DIM);
	mpf_y_v = init_mpfvector(DIM);
	for(i = 0; i < DIM; i++)
	{
//		mpf_init_set_ui(mpf_y0[i], 1UL);
		mpf_init_set_ui(mpf_y0[i], 0UL); // for ex3 
		smpfvi(mpf_y0_v, i, mpf_y0[i]);
		mpf_init(mpf_y[i]);
	}
	fflush(stdout); printf("mpf_ivp\n");

	goto mpf_nim;

	/* y' = y */
	mpf_set_str(mpf_tmp, "0.05", 10); ivp_mpftaylor_fstep(mpf_y, mpf_x0, mpf_y0, mpf_df3, mpf_tmp, 40, DIM);
//	mpf_set_str(mpf_tmp, "0.1", 10); ivp_mpftaylor_fstep(mpf_y, mpf_x0, mpf_y0, mpf_df2, mpf_tmp, 10, DIM);
//	mpf_set_str(mpf_tmp, "1", 10); ivp_mpftaylor_fstep(mpf_y, mpf_x0, mpf_y0, mpf_df1, mpf_tmp, 10, DIM);
//	for(i = 0; i < DIM; i++)
//		printf("%5d: %25.17e %25.17e\n", i, mpf_get_d(mpf_y[i]), ans3(1.0));
//		printf("%5d: %25.17e %25.17e\n", i, mpf_get_d(mpf_y[i]), ans2(1.0));
//		printf("%5d: %25.17e %25.17e\n", i, mpf_get_d(mpf_y[i]), ans1(10.0));
	for(i = 0; i < DIM; i++)
	{
		printf("%5d: ", i); mpf_out_str(stdout, 10, 0, mpf_y[i]); printf(", "); mpf_set_str(mpf_tmp, "-10", 10); mpf_exp(mpf_y[i], mpf_tmp); mpf_out_str(stdout, 10, 0, mpf_y[i]); printf("\n");
	}

goto end;

	mpf_set_str(mpf_tmp, "2", 10); mpf_erk_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 40, mpf_df3_1, ERK_BUTCHER1_7_6);
//	mpf_set_str(mpf_tmp, "1", 10); mpf_erk_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 10, mpf_df2_1, ERK_BUTCHER1_7_6);
//	mpf_set_str(mpf_tmp, "10", 10); mpf_erk_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 10, mpf_df1_1, ERK_BUTCHER1_7_6);

//	goto end;
mpf_nim:
	mpf_set_str(mpf_tmp1, "0", 10);
	mpf_set_str(mpf_tmp, "2", 10); mpf_ex_nim_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 40, mpf_df3_1, mpf_tmp1, mpf_tmp1, 8);
//	mpf_set_str(mpf_tmp, "1", 10); mpf_ex_nim_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 10, mpf_df2_1, mpf_tmp1, mpf_tmp1, 8);
//	mpf_set_str(mpf_tmp, "10", 10); mpf_ex_nim_fs(stdout, mpf_tmp, mpf_y_v, mpf_x0, mpf_y0_v, 10, mpf_df1_1, mpf_tmp1, mpf_tmp1, 8);


	mpf_clear(mpf_x0);
	mpf_clear(mpf_tmp);
	mpf_clear(mpf_tmp1);
	free_mpfvector(mpf_y0_v);
	free_mpfvector(mpf_y_v);

end:
	return 0;
}
