/********************************************************************************/
/* linear_append.c: Newer Additional Functions to linear.c                      */
/* Copyright (c) 2012 Tomonori Kouya                                            */
/*                                                                              */
/* Version 0.1, 2012-03-18: Newly appended to BNCpack                           */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
#include "bnc.h"

//#include "bnclinear_append.h"

/* c := (double)a */
void subst_fmatrix_dmat(FMatrix c, DMatrix a)
{
	long int i, j;
	float tmp;

	if((c->row_dim != a->row_dim) || (c->col_dim != a->col_dim))
	{
		fprintf(stderr, "ERROR: subst_fmatrix_dmat\n");
		return;
	}

	for(i = 0; i < a->row_dim; i++)
	{
		for(j = 0; j < a->col_dim; j++)
		{
			set_fmatrix_ij(c, i, j, (float)get_dmatrix_ij(a, i, j));
		}
	}
}

/* c := (double)a^T */
void subst_fmatrix_dmat_trans(FMatrix c, DMatrix a)
{
	long int i, j;
	float tmp;

	if((c->row_dim != a->col_dim) || (c->col_dim != a->row_dim))
	{
		fprintf(stderr, "ERROR: subst_fmatrix_dmat_trans\n");
		return;
	}

	for(i = 0; i < a->row_dim; i++)
	{
		for(j = 0; j < a->col_dim; j++)
		{
			set_fmatrix_ij(c, j, i, (float)get_dmatrix_ij(a, i, j));
		}
	}
}

/* c := (double)a */
void subst_fvector_dvec(FVector c, DVector a)
{
	long int i;

	for(i = 0; i < a->dim; i++)
		set_fvector_i(c, i, (float)get_dvector_i(a, i));
}

/* (double)c := (float)a */
void subst_dvector_fvec(DVector c, FVector a)
{
	long int i;

	for(i = 0; i < a->dim; i++)
		set_dvector_i(c, i, (double)get_fvector_i(a, i));
}

/* get residual in double precision */
/* r := b - A * x */
void residual_dmat_dvec(DVector r, DVector b, DMatrix mat, DVector x)
{
	mul_dmatrix_dvec(r, mat, x);
	sub_dvector(r, b, r);
}

/* vec := -vec */
void neg_dvector(DVector vec)
{
	long int i;
	double tmp;

	for(i = 0; i < vec->dim; i++)
	{
		tmp = get_dvector_i(vec, i);
		set_dvector_i(vec, i, -tmp);
	}
}

/* Elementwise relative errors of vector */
void relerr_element_dvector(double *max_relerr, double *min_relerr, double *norm_relerr, DVector approx_vec, DVector true_vec, int norm_type)
{
	long int i;
	double norm_diff_vec, norm_true_vec;
	double abs_true_vec, abs_diff_vec;
	DVector diff_vec;

	diff_vec = init_dvector(approx_vec->dim);

	// diff_vec := approx_vec - true_vec
	sub_dvector(diff_vec, approx_vec, true_vec);

	switch(norm_type)
	{
		// inifinity norm 
		case 0:
			norm_diff_vec = normi_dvector(diff_vec);
			norm_true_vec = normi_dvector(true_vec);
			break;
		// 1-norm
		case 1:
			norm_diff_vec = norm1_dvector(diff_vec);
			norm_true_vec = norm1_dvector(true_vec);
			break;

		// euclid norm: default
		case 2:
		default:
			norm_diff_vec = norm2_dvector(diff_vec);
			norm_true_vec = norm2_dvector(true_vec);
			break;
	}

	if(norm_true_vec != 0.0)
		*norm_relerr = norm_diff_vec / norm_true_vec;

	// relative errors of each elements
	*max_relerr = 0.0;
	*min_relerr = normi_dvector(diff_vec);
	for(i = 0; i < approx_vec->dim; i++)
	{
		abs_diff_vec = fabs(get_dvector_i(diff_vec, i));
		abs_true_vec = fabs(get_dvector_i(true_vec, i));
		if(abs_true_vec != 0.0)
			abs_diff_vec /= abs_true_vec;
		
		if(*max_relerr < abs_diff_vec)
			*max_relerr = abs_diff_vec;
		if(*min_relerr > abs_diff_vec)
			*min_relerr = abs_diff_vec;
	}

	free_dvector(diff_vec);

	return;
}

#ifdef USE_GMP

/* get residual in mpf_t precision */
/* r := b - A * x */
void residual_mpfmat_mpfvec(MPFVector r, MPFVector b, MPFMatrix mat, MPFVector x)
{
	mul_mpfmatrix_mpfvec(r, mat, x);
	sub_mpfvector(r, b, r);
}

/* c := (mpf_t)a */
void subst_dmatrix_mpfmat(DMatrix c, MPFMatrix a)
{
	long int i, j;
	double tmp;

	if((c->row_dim != a->row_dim) || (c->col_dim != a->col_dim))
	{
		fprintf(stderr, "ERROR: subst_dmatrix_mpfmat\n");
		return;
	}

	for(i = 0; i < a->row_dim; i++)
	{
		for(j = 0; j < a->col_dim; j++)
		{
			tmp = mpf_get_d(get_mpfmatrix_ij(a, i, j));
			set_dmatrix_ij(c, i, j, tmp);
		}
	}
}

/* c := (mpf_t)a^T */
void subst_dmatrix_mpfmat_trans(DMatrix c, MPFMatrix a)
{
	long int i, j;
	double tmp;

	if((c->row_dim != a->col_dim) || (c->col_dim != a->row_dim))
	{
		fprintf(stderr, "ERROR: subst_dmatrix_mpfmat_trans\n");
		return;
	}

	for(i = 0; i < a->row_dim; i++)
	{
		for(j = 0; j < a->col_dim; j++)
		{
			tmp = mpf_get_d(get_mpfmatrix_ij(a, i, j));
			set_dmatrix_ij(c, j, i, tmp);
		}
	}
}

/* c := (double)a */
void subst_dvector_mpfvec(DVector c, MPFVector a)
{
	long int i;
	double tmp;

	for(i = 0; i < a->dim; i++)
	{
		tmp = mpf_get_d(get_mpfvector_i(a, i));
		set_dvector_i(c, i, tmp);
	}
}

/* (mpf_t)c := (double)a */
void subst_mpfvector_dvec(MPFVector c, DVector a)
{
	long int i;
	mpf_t tmp;

	mpf_init2(tmp, c->prec);

	for(i = 0; i < a->dim; i++)
	{
		mpf_set_d(tmp, get_dvector_i(a, i));
		set_mpfvector_i(c, i, tmp);
	}

	mpf_clear(tmp);
}

/* vec := -vec */
void neg_mpfvector(MPFVector vec)
{
	long int i;
	mpf_t tmp;

	mpf_init2(tmp, vec->prec);

	for(i = 0; i < vec->dim; i++)
	{
		mpf_neg(tmp, get_mpfvector_i(vec, i));
		set_mpfvector_i(vec, i, tmp);
	}

	mpf_clear(tmp);
}

/* Normwise relative error of vector */
void relerr_mpfvector(mpf_t relerr, MPFVector approx_vec, MPFVector true_vec, int norm_type)
{
	unsigned long prec;
	mpf_t norm_true_vec, norm_diff_vec;
	MPFVector diff_vec;

	prec = approx_vec->prec;

	diff_vec = init2_mpfvector(approx_vec->dim, prec);
	mpf_init2(norm_true_vec, prec);
	mpf_init2(norm_diff_vec, prec);

	// diff_vec := approx_vec - true_vec
	sub_mpfvector(diff_vec, approx_vec, true_vec);

	switch(norm_type)
	{
		// inifinity norm 
		case 0:
			normi_mpfvector(norm_diff_vec, diff_vec);
			normi_mpfvector(norm_true_vec, true_vec);
			break;

		// 1-norm
		case 1:
			norm1_mpfvector(norm_diff_vec, diff_vec);
			norm1_mpfvector(norm_true_vec, true_vec);
			break;

		// euclid norm: default
		case 2:
		default:
			norm2_mpfvector(norm_diff_vec, diff_vec);
			norm2_mpfvector(norm_true_vec, true_vec);
			break;
	}

	if(mpf_cmp_ui(norm_true_vec, 0UL) != 0)
		mpf_div(relerr, norm_diff_vec, norm_true_vec);

	free_mpfvector(diff_vec);
	mpf_clear(norm_diff_vec);
	mpf_clear(norm_true_vec);

	return;
}

/* Elementwise relative errors of vector */
void relerr_element_mpfvector(mpf_t max_relerr, mpf_t min_relerr, mpf_t norm_relerr, MPFVector approx_vec, MPFVector true_vec, int norm_type)
{
	unsigned long prec;
	long int i;
	mpf_t abs_true_vec, abs_diff_vec, norm_diff_vec, norm_true_vec;
	MPFVector diff_vec;

	prec = approx_vec->prec;

	diff_vec = init2_mpfvector(approx_vec->dim, prec);
	mpf_init2(abs_true_vec, prec);
	mpf_init2(abs_diff_vec, prec);
	mpf_init2(norm_diff_vec, prec);
	mpf_init2(norm_true_vec, prec);

	// diff_vec := approx_vec - true_vec
	sub_mpfvector(diff_vec, approx_vec, true_vec);

	switch(norm_type)
	{
		// inifinity norm 
		case 0:
			normi_mpfvector(norm_diff_vec, diff_vec);
			normi_mpfvector(norm_true_vec, true_vec);
			break;

		// 1-norm
		case 1:
			norm1_mpfvector(norm_diff_vec, diff_vec);
			norm1_mpfvector(norm_true_vec, true_vec);
			break;

		// euclid norm: default
		case 2:
		default:
			norm2_mpfvector(norm_diff_vec, diff_vec);
			norm2_mpfvector(norm_true_vec, true_vec);
			break;
	}

	mpf_set(norm_relerr, norm_diff_vec);
	if(mpf_cmp_ui(norm_true_vec, 0UL) != 0)
		mpf_div(norm_relerr, norm_diff_vec, norm_true_vec);

	// relative errors of each elements
	mpf_set_ui(max_relerr, 0UL);
	normi_mpfvector(min_relerr, diff_vec);
	for(i = 0; i < approx_vec->dim; i++)
	{
		mpf_abs(abs_diff_vec, get_mpfvector_i(diff_vec, i));
		mpf_abs(abs_true_vec, get_mpfvector_i(true_vec, i));
		if(mpf_cmp_ui(abs_true_vec, 0UL) != 0)
			mpf_div(abs_diff_vec, abs_diff_vec, abs_true_vec);
		
		if(mpf_cmp(max_relerr, abs_diff_vec) < 0)
			mpf_set(max_relerr, abs_diff_vec);
		if(mpf_cmp(min_relerr, abs_diff_vec) > 0)
			mpf_set(min_relerr, abs_diff_vec);
	}

	free_mpfvector(diff_vec);// Fix! 2012-06-03 by T.Kouya
	mpf_clear(abs_true_vec);
	mpf_clear(abs_diff_vec);
	mpf_clear(norm_true_vec);
	mpf_clear(norm_diff_vec);

	return;
}
#endif
