/********************************************************************************/
/* test_complex.c:                                                              */
/* Copyright (C) 2003-2011 Tomonori Kouya                                       */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
/* Test for complex.c */

#include <stdio.h>
#include <math.h>
#include "bnc.h"

main()
{
	FCmplx fca, fcb, fcc;
	DCmplx dca, dcb, dcc;
#ifdef USE_GMP
	MPFCmplx mpfca, mpfcb, mpfcc;
	mpf_t tmp;
#endif

/* float */

	/* init */
	fca = init_fcmplx();
	fcb = init_fcmplx();
	fcc = init_fcmplx();

	/* set */
	set_real_fcmplx(fca, (float)1.0);
	set_image_fcmplx(fca, (float)2.0);
	set_real_fcmplx(fcb, (float)3.0);
	set_image_fcmplx(fcb, (float)4.0);
	set_real_fcmplx(fcc, (float)rand());
	set_image_fcmplx(fcc, (float)rand());

	/* print */
	printf("fca = "); print_fcmplx(fca);
	printf("fcb = "); print_fcmplx(fcb);
	printf("fcc = "); print_fcmplx(fcc);

	/* basic arithmetic */
	add_fcmplx(fcc, fca, fcb); printf("fca + fcb = "); print_fcmplx(fcc);
	sub_fcmplx(fcc, fca, fcb); printf("fca - fcb = "); print_fcmplx(fcc);
	add2_fcmplx(fcc, fcb); printf("fca - fcb + fcb = "); print_fcmplx(fcc);

	mul_fcmplx(fcc, fca, fcb); printf("fca * fcb = "); print_fcmplx(fcc);
	div_fcmplx(fcc, fca, fcb); printf("fca / fcb = "); print_fcmplx(fcc);
	mul2_fcmplx(fcc, fcb); printf("fca / fcb * fcb = "); print_fcmplx(fcc);

	printf("~fca  = "); conj_fcmplx(fcc, fca); print_fcmplx(fcc);
	printf("|fca| = %15.7e\n", abs_fcmplx(fca));
	printf("exp(i*fca) = "); iexp_fcmplx(fcc, abs_fcmplx(fca)); print_fcmplx(fcc);

	/* clear */
	free_fcmplx(fca);
	free_fcmplx(fcb);
	free_fcmplx(fcc);

/* double */

	/* init */
	dca = init_dcmplx();
	dcb = init_dcmplx();
	dcc = init_dcmplx();

	/* set */
	set_real_dcmplx(dca, 1.0);
	set_image_dcmplx(dca, 2.0);
	set_real_dcmplx(dcb, 3.0);
	set_image_dcmplx(dcb, 4.0);
	set_real_dcmplx(dcc, (double)rand());
	set_image_dcmplx(dcc, (double)rand());

	/* print */
	printf("dca = "); print_dcmplx(dca);
	printf("dcb = "); print_dcmplx(dcb);
	printf("dcc = "); print_dcmplx(dcc);

	/* basic arithmetic */
	add_dcmplx(dcc, dca, dcb); printf("dca + dcb = "); print_dcmplx(dcc);
	sub_dcmplx(dcc, dca, dcb); printf("dca - dcb = "); print_dcmplx(dcc);
	add2_dcmplx(dcc, dcb); printf("dca - dcb + dcb = "); print_dcmplx(dcc);

	mul_dcmplx(dcc, dca, dcb); printf("dca * dcb = "); print_dcmplx(dcc);
	div_dcmplx(dcc, dca, dcb); printf("dca / dcb = "); print_dcmplx(dcc);
	mul2_dcmplx(dcc, dcb); printf("dca / dcb * dcb = "); print_dcmplx(dcc);

	printf("~dca  = "); conj_dcmplx(dcc, dca); print_dcmplx(dcc);
	printf("|dca| = %25.17e\n", abs_dcmplx(dca));
	printf("exp(i*dca) = "); iexp_dcmplx(dcc, abs_dcmplx(dca)); print_dcmplx(dcc);

	/* clear */
	free_dcmplx(dca);
	free_dcmplx(dcb);
	free_dcmplx(dcc);

#ifdef USE_GMP
/* mpf_t */

	/* init */
	set_bnc_default_prec(128);
	mpfca = init_mpfcmplx();
	mpfcb = init_mpfcmplx();
	mpfcc = init2_mpfcmplx(256);

	/* set */
	set_real_mpfcmplx_ui(mpfca, 1);
	set_image_mpfcmplx_ui(mpfca, 2);
	set_real_mpfcmplx_ui(mpfcb, 3);
	set_image_mpfcmplx_ui(mpfcb, 4);
	set_real_mpfcmplx_ui(mpfcc, rand());
	set_image_mpfcmplx_ui(mpfcc, rand());

	/* print */
	printf("mpfca = "); print_mpfcmplx(mpfca);
	printf("mpfcb = "); print_mpfcmplx(mpfcb);
	printf("mpfcc = "); print_mpfcmplx(mpfcc);

	/* basic arithmetic */
	add_mpfcmplx(mpfcc, mpfca, mpfcb); printf("mpfca + mpfcb = "); print_mpfcmplx(mpfcc);
	sub_mpfcmplx(mpfcc, mpfca, mpfcb); printf("mpfca - mpfcb = "); print_mpfcmplx(mpfcc);
	add2_mpfcmplx(mpfcc, mpfcb); printf("mpfca - mpfcb + mpfcb = "); print_mpfcmplx(mpfcc);

	mul_mpfcmplx(mpfcc, mpfca, mpfcb); printf("mpfca * mpfcb = "); print_mpfcmplx(mpfcc);
	div_mpfcmplx(mpfcc, mpfca, mpfcb); printf("mpfca / mpfcb = "); print_mpfcmplx(mpfcc);
	mul2_mpfcmplx(mpfcc, mpfcb); printf("mpfca / mpfcb * mpfcb = "); print_mpfcmplx(mpfcc);

	printf("~mpfca  = "); conj_mpfcmplx(mpfcc, mpfca); print_mpfcmplx(mpfcc);
	mpf_init(tmp); abs_mpfcmplx(tmp, mpfca);
	printf("|mpfca| = "); mpf_out_str(stdout, 10, 0, tmp); printf("\n");
	printf("exp(i*mpfca) = "); iexp_mpfcmplx(mpfcc, tmp); print_mpfcmplx(mpfcc);
	mpf_clear(tmp);

	/* clear */
	free_mpfcmplx(mpfca);
	free_mpfcmplx(mpfcb);
	free_mpfcmplx(mpfcc);
#endif
}
