/********************************************************************************/
/* test_gtestmat.c:                                                             */
/* Copyright (C) 2003-2011 Tomonori Kouya                                       */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
/*************************************************/
/* Test Program for gtestmat.c                   */
/*************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "bnc.h"

#define DIM 5

main()
{
	DMatrix da, db, dc;
	DVector dv, dv1, dv2;
	long int i;
#ifdef USE_GMP
	MPFMatrix mpfa, mpfb, mpfc;
	MPFVector mpfv, mpfv1, mpfv2;
	mpf_t tmp;
#endif

/* Double */
	/* initialize */
	da = init_dmatrix(DIM, DIM);
	db = init_dmatrix(DIM, DIM);
	dc = init_dmatrix(DIM, DIM);
	dv = init_dvector(DIM);
	dv1 = init_dvector(DIM);
	dv2 = init_dvector(DIM);

	/* get test matrix */
	hilbert_dmatrix(da, DIM); print_dmatrix(da); printf("\n");
	lotkin_dmatrix(da, DIM); print_dmatrix(da); printf("\n");
	frank_dmatrix(da, DIM); print_dmatrix(da); printf("\n");
	subst_dmatrix(db, da); inv_dmatrix(da); print_dmatrix(da);

	int_sym_rand_dmatrix(da, DIM, 1, DIM); print_dmatrix(da); printf("\n");
	int_unsym_rand_dmatrix(da, DIM, 1, DIM); print_dmatrix(da); printf("\n");

	for(i = 0; i < DIM; i++)
		set_dvector_i(dv, i, (double)(i + 1));
	diag_dmatrix(da, dv, DIM); print_dmatrix(da); printf("\n");

	for(i = 0; i < DIM; i++)
	{
		set_dvector_i(dv, i, 2.0);
		set_dvector_i(dv1, i, -1.0);
		set_dvector_i(dv2, i, -1.0);
	}
	set_dvector_i(dv, 0, 1.0);
	tridiag_dmatrix(db, dv1, dv, dv2, DIM); print_dmatrix(db); printf("\n");
	subst_dmatrix(da, db); inv_dmatrix(da); print_dmatrix(da);

	/* Clear */
	free_dmatrix(da);
	free_dmatrix(db);
	free_dmatrix(dc);
	free_dvector(dv);
	free_dvector(dv1);
	free_dvector(dv2);

/* MPF */
#ifdef USE_GMP
	set_bnc_default_prec(128);

	/* initialize */
	mpfa = init_mpfmatrix(DIM, DIM);
	mpfb = init_mpfmatrix(DIM, DIM);
	mpfc = init_mpfmatrix(DIM, DIM);
	mpfv = init_mpfvector(DIM);
	mpfv1 = init_mpfvector(DIM);
	mpfv2 = init_mpfvector(DIM);

	/* get test matrix */
	hilbert_mpfmatrix(mpfa, DIM); print_mpfmatrix(mpfa); printf("\n");
	lotkin_mpfmatrix(mpfa, DIM); print_mpfmatrix(mpfa); printf("\n");
	frank_mpfmatrix(mpfa, DIM); print_mpfmatrix(mpfa); printf("\n");
	subst_mpfmatrix(mpfb, mpfa); inv_mpfmatrix(mpfa); print_mpfmatrix(mpfa);

	int_sym_rand_mpfmatrix(mpfa, DIM, 1, DIM); print_mpfmatrix(mpfa); printf("\n");
	int_unsym_rand_mpfmatrix(mpfa, DIM, 1, DIM); print_mpfmatrix(mpfa); printf("\n");

	mpf_init(tmp);
	for(i = 0; i < DIM; i++)
	{
		mpf_set_ui(tmp, (unsigned long)(i + 1));
		set_mpfvector_i(mpfv, i, tmp);
	}
	diag_mpfmatrix(mpfa, mpfv, DIM); print_mpfmatrix(mpfa); printf("\n");

	for(i = 0; i < DIM; i++)
	{
		mpf_set_ui(tmp, 2UL);
		set_mpfvector_i(mpfv, i, tmp);
		mpf_set_si(tmp, -1);
		set_mpfvector_i(mpfv1, i, tmp);
		set_mpfvector_i(mpfv2, i, tmp);
	}
	mpf_set_ui(tmp, 1UL);
	set_mpfvector_i(mpfv, 0, tmp);
	tridiag_mpfmatrix(mpfb, mpfv1, mpfv, mpfv2, DIM); print_mpfmatrix(mpfb); printf("\n");
	subst_mpfmatrix(mpfa, mpfb); inv_mpfmatrix(mpfa); print_mpfmatrix(mpfa);

	/* Clear */
	mpf_clear(tmp);
	free_mpfmatrix(mpfa);
	free_mpfmatrix(mpfb);
	free_mpfmatrix(mpfc);
	free_mpfvector(mpfv);
	free_mpfvector(mpfv1);
	free_mpfvector(mpfv2);
#endif
}

