/********************************************************************************/
/* test_linear.c:                                                               */
/* Copyright (C) 2003-2011 Tomonori Kouya                                       */
/*                                                                              */
/* This program is free software: you can redistribute it and/or modify it      */
/* under the terms of the GNU Lesser General Public License as published by the */
/* Free Software Foundation, either version 3 of the License or any later       */
/* version.                                                                     */
/*                                                                              */
/* This program is distributed in the hope that it will be useful, but WITHOUT  */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        */
/* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License */
/* for more details.                                                            */
/*                                                                              */
/* You should have received a copy of the GNU Lesser General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                              */
/********************************************************************************/
/*************************************************/
/* For Debug                                     */
/*************************************************/

#include <stdio.h>
#include <math.h>

#include "bnc.h"

main()
{
	FVector fvec;
	DVector dvec;
	FMatrix fmat, fmat_a, fmat_b;
	DMatrix dmat;
#ifdef USE_GMP
	MPFVector mpfvec;
	MPFMatrix mpfmat;
#endif

	long int dim, row_dim, col_dim, i, j;

#ifdef USE_GMP
	set_bnc_default_prec(128);
#endif

	/* For Vectors */
	for(dim = 10; dim <= 1000; dim *= 10)
	{
		fvec = init_fvector(dim);
		dvec = init_dvector(dim);
#ifdef USE_GMP
		mpfvec = init_mpfvector(dim);
		if(mpfvec == NULL)
			exit(0);
#endif
		
#ifdef USE_GMP
		print_fdmpfvector(fvec, dvec, mpfvec);
#else
		print_fdvector(fvec, dvec);
#endif

		for(i = 0; i < dim; i++)
		{
			set_fvector_i(fvec, i, (float)(i + 1));
			set_dvector_i(dvec, i, (double)(i + 1));
			printf("test2\n"); fflush(stdout);
#ifdef USE_GMP
			set_mpfvector_i_d(mpfvec, i, (double)(i + 1));
#endif
		}

#ifdef USE_GMP
		print_fdmpfvector(fvec, dvec, mpfvec);
#else
		print_fdvector(fvec, dvec);
#endif

		free_fvector(fvec);
		free_dvector(dvec);
#ifdef USE_GMP
	printf("test3\n"); fflush(stdout);
		free_mpfvector(mpfvec);
#endif

	}

	/* For Matrices */
	for(row_dim = 1; row_dim <= 1024; row_dim *= 2)
	{
		for(col_dim = 1; col_dim <= 1024; col_dim *= 2)
		{
			fmat   = init_fmatrix(row_dim, col_dim);
			fmat_a = init_fmatrix(row_dim, col_dim);
			fmat_b = init_fmatrix(row_dim, col_dim);

			dmat = init_dmatrix(row_dim, col_dim);
#ifdef USE_GMP
			mpfmat = init_mpfmatrix(row_dim, col_dim);
#endif

			for(i = 0; i < row_dim; i++)
			{
				for(j = 0; j < col_dim; j++)
				{
					set_fmatrix_ij(fmat, i, j, (float)(i + j + 1));
					set_dmatrix_ij(dmat, i, j, (double)(i + j + 1));
#ifdef USE_GMP
					set_mpfmatrix_ij_d(mpfmat, i, j, (double)(i + j + 1));
#endif
					//printf("%10.3f %10.3f\n", get_fmatrix_ij(fmat, i, j), get_dmatrix_ij(dmat, i, j));
					printf("(%3ld, %3ld) %10.3f %10.3f\n", i, j, get_dmatrix_ij(dmat, i, j), *(dmat->element + i * dmat->col_dim  + j));
				}
			}
			printf("fmat:\n");
			print_fmatrix(fmat);
			subst_fmatrix(fmat_a, fmat);
			printf("fmat_inv:\n");
			inv_fmatrix(fmat_a);
			print_fmatrix(fmat_a);
			mul_fmatrix(fmat_b, fmat, fmat_a);
			print_fmatrix(fmat_b);

			printf("dmat:\n");
			print_dmatrix(dmat);
			for(i = 0; i < dmat->row_dim; i++)
				for(j = 0; j < dmat->col_dim; j++)
					printf("(%3ld, %3ld) %10.3f %10.3f\n", i, j, get_dmatrix_ij(dmat, i, j), *(dmat->element + (i * dmat->col_dim  + j)));

#ifdef USE_GMP
			printf("mpfmat:\n");
			print_mpfmatrix(mpfmat);
#endif

			free_fmatrix(fmat);
			free_fmatrix(fmat_a);
			free_fmatrix(fmat_b);

			free_dmatrix(dmat);
#ifdef USE_GMP
			free_mpfmatrix(mpfmat);
#endif

			printf("\n");
		}
	}
}
